# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: harness/wire.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='harness/wire.proto',
  package='harness',
  syntax='proto2',
  serialized_options=None,
  serialized_pb=b'\n\x12harness/wire.proto\x12\x07harness\x1a google/protobuf/descriptor.proto\x1a\x17validate/validate.proto\"Y\n\x04Mark\x12(\n\x08protocol\x18\x01 \x01(\x0e\x32\x16.harness.Mark.Protocol\"\'\n\x08Protocol\x12\x07\n\x03TCP\x10\x00\x12\x08\n\x04HTTP\x10\x01\x12\x08\n\x04GRPC\x10\x02\"\x85\x01\n\x05Input\x12\x15\n\x04type\x18\x01 \x02(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12#\n\x05reach\x18\x02 \x01(\x0e\x32\x14.harness.Input.Reach\"@\n\x05Reach\x12\r\n\tLOCALHOST\x10\x00\x12\r\n\tNAMESPACE\x10\x01\x12\x0b\n\x07\x43LUSTER\x10\x02\x12\x0c\n\x08\x45XTERNAL\x10\x03\"\x86\x01\n\x06Output\x12\x15\n\x04type\x18\x01 \x02(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12&\n\x06\x65xpose\x18\x02 \x01(\x0e\x32\x16.harness.Output.Expose\"=\n\x06\x45xpose\x12\x0b\n\x07PRIVATE\x10\x00\x12\x0c\n\x08HEADLESS\x10\x01\x12\x0c\n\x08INTERNAL\x10\x02\x12\n\n\x06PUBLIC\x10\x03\"W\n\x04Wire\x12\x1f\n\x05input\x18\x01 \x01(\x0b\x32\x0e.harness.InputH\x00\x12!\n\x06output\x18\x02 \x01(\x0b\x32\x0f.harness.OutputH\x00\x42\x0b\n\x04type\x12\x03\xf8\x42\x01\"\xa4\x02\n\x07Service\x12\x0c\n\x04name\x18\x01 \x01(\t\x12-\n\tcontainer\x18\x02 \x01(\x0b\x32\x1a.harness.Service.Container\x1a\'\n\x08Resource\x12\x0b\n\x03\x63pu\x18\x01 \x01(\t\x12\x0e\n\x06memory\x18\x02 \x01(\t\x1a\x63\n\tResources\x12+\n\x08requests\x18\x01 \x01(\x0b\x32\x19.harness.Service.Resource\x12)\n\x06limits\x18\x02 \x01(\x0b\x32\x19.harness.Service.Resource\x1aN\n\tContainer\x12\x12\n\nrepository\x18\x01 \x01(\t\x12-\n\tresources\x18\x02 \x01(\x0b\x32\x1a.harness.Service.Resources:;\n\x04wire\x12\x1d.google.protobuf.FieldOptions\x18\xd1\x0f \x01(\x0b\x32\r.harness.Wire:;\n\x04mark\x12\x1d.google.protobuf.FieldOptions\x18\xd2\x0f \x01(\x0b\x32\r.harness.Mark:C\n\x07service\x12\x1f.google.protobuf.MessageOptions\x18\xd1\x0f \x01(\x0b\x32\x10.harness.Service'
  ,
  dependencies=[google_dot_protobuf_dot_descriptor__pb2.DESCRIPTOR,validate_dot_validate__pb2.DESCRIPTOR,])


WIRE_FIELD_NUMBER = 2001
wire = _descriptor.FieldDescriptor(
  name='wire', full_name='harness.wire', index=0,
  number=2001, type=11, cpp_type=10, label=1,
  has_default_value=False, default_value=None,
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  serialized_options=None, file=DESCRIPTOR)
MARK_FIELD_NUMBER = 2002
mark = _descriptor.FieldDescriptor(
  name='mark', full_name='harness.mark', index=1,
  number=2002, type=11, cpp_type=10, label=1,
  has_default_value=False, default_value=None,
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  serialized_options=None, file=DESCRIPTOR)
SERVICE_FIELD_NUMBER = 2001
service = _descriptor.FieldDescriptor(
  name='service', full_name='harness.service', index=2,
  number=2001, type=11, cpp_type=10, label=1,
  has_default_value=False, default_value=None,
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  serialized_options=None, file=DESCRIPTOR)

_MARK_PROTOCOL = _descriptor.EnumDescriptor(
  name='Protocol',
  full_name='harness.Mark.Protocol',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TCP', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HTTP', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GRPC', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=140,
  serialized_end=179,
)
_sym_db.RegisterEnumDescriptor(_MARK_PROTOCOL)

_INPUT_REACH = _descriptor.EnumDescriptor(
  name='Reach',
  full_name='harness.Input.Reach',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='LOCALHOST', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NAMESPACE', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CLUSTER', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='EXTERNAL', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=251,
  serialized_end=315,
)
_sym_db.RegisterEnumDescriptor(_INPUT_REACH)

_OUTPUT_EXPOSE = _descriptor.EnumDescriptor(
  name='Expose',
  full_name='harness.Output.Expose',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PRIVATE', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HEADLESS', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INTERNAL', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PUBLIC', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=391,
  serialized_end=452,
)
_sym_db.RegisterEnumDescriptor(_OUTPUT_EXPOSE)


_MARK = _descriptor.Descriptor(
  name='Mark',
  full_name='harness.Mark',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='protocol', full_name='harness.Mark.protocol', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _MARK_PROTOCOL,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=90,
  serialized_end=179,
)


_INPUT = _descriptor.Descriptor(
  name='Input',
  full_name='harness.Input',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='harness.Input.type', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\004r\002\020\001', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='reach', full_name='harness.Input.reach', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _INPUT_REACH,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=182,
  serialized_end=315,
)


_OUTPUT = _descriptor.Descriptor(
  name='Output',
  full_name='harness.Output',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='harness.Output.type', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\004r\002\020\001', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='expose', full_name='harness.Output.expose', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _OUTPUT_EXPOSE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=318,
  serialized_end=452,
)


_WIRE = _descriptor.Descriptor(
  name='Wire',
  full_name='harness.Wire',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='input', full_name='harness.Wire.input', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='output', full_name='harness.Wire.output', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='type', full_name='harness.Wire.type',
      index=0, containing_type=None, fields=[], serialized_options=b'\370B\001'),
  ],
  serialized_start=454,
  serialized_end=541,
)


_SERVICE_RESOURCE = _descriptor.Descriptor(
  name='Resource',
  full_name='harness.Service.Resource',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='cpu', full_name='harness.Service.Resource.cpu', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='memory', full_name='harness.Service.Resource.memory', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=616,
  serialized_end=655,
)

_SERVICE_RESOURCES = _descriptor.Descriptor(
  name='Resources',
  full_name='harness.Service.Resources',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='requests', full_name='harness.Service.Resources.requests', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='limits', full_name='harness.Service.Resources.limits', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=657,
  serialized_end=756,
)

_SERVICE_CONTAINER = _descriptor.Descriptor(
  name='Container',
  full_name='harness.Service.Container',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='repository', full_name='harness.Service.Container.repository', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='resources', full_name='harness.Service.Container.resources', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=758,
  serialized_end=836,
)

_SERVICE = _descriptor.Descriptor(
  name='Service',
  full_name='harness.Service',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='harness.Service.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='container', full_name='harness.Service.container', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_SERVICE_RESOURCE, _SERVICE_RESOURCES, _SERVICE_CONTAINER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=544,
  serialized_end=836,
)

_MARK.fields_by_name['protocol'].enum_type = _MARK_PROTOCOL
_MARK_PROTOCOL.containing_type = _MARK
_INPUT.fields_by_name['reach'].enum_type = _INPUT_REACH
_INPUT_REACH.containing_type = _INPUT
_OUTPUT.fields_by_name['expose'].enum_type = _OUTPUT_EXPOSE
_OUTPUT_EXPOSE.containing_type = _OUTPUT
_WIRE.fields_by_name['input'].message_type = _INPUT
_WIRE.fields_by_name['output'].message_type = _OUTPUT
_WIRE.oneofs_by_name['type'].fields.append(
  _WIRE.fields_by_name['input'])
_WIRE.fields_by_name['input'].containing_oneof = _WIRE.oneofs_by_name['type']
_WIRE.oneofs_by_name['type'].fields.append(
  _WIRE.fields_by_name['output'])
_WIRE.fields_by_name['output'].containing_oneof = _WIRE.oneofs_by_name['type']
_SERVICE_RESOURCE.containing_type = _SERVICE
_SERVICE_RESOURCES.fields_by_name['requests'].message_type = _SERVICE_RESOURCE
_SERVICE_RESOURCES.fields_by_name['limits'].message_type = _SERVICE_RESOURCE
_SERVICE_RESOURCES.containing_type = _SERVICE
_SERVICE_CONTAINER.fields_by_name['resources'].message_type = _SERVICE_RESOURCES
_SERVICE_CONTAINER.containing_type = _SERVICE
_SERVICE.fields_by_name['container'].message_type = _SERVICE_CONTAINER
DESCRIPTOR.message_types_by_name['Mark'] = _MARK
DESCRIPTOR.message_types_by_name['Input'] = _INPUT
DESCRIPTOR.message_types_by_name['Output'] = _OUTPUT
DESCRIPTOR.message_types_by_name['Wire'] = _WIRE
DESCRIPTOR.message_types_by_name['Service'] = _SERVICE
DESCRIPTOR.extensions_by_name['wire'] = wire
DESCRIPTOR.extensions_by_name['mark'] = mark
DESCRIPTOR.extensions_by_name['service'] = service
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Mark = _reflection.GeneratedProtocolMessageType('Mark', (_message.Message,), {
  'DESCRIPTOR' : _MARK,
  '__module__' : 'harness.wire_pb2'
  # @@protoc_insertion_point(class_scope:harness.Mark)
  })
_sym_db.RegisterMessage(Mark)

Input = _reflection.GeneratedProtocolMessageType('Input', (_message.Message,), {
  'DESCRIPTOR' : _INPUT,
  '__module__' : 'harness.wire_pb2'
  # @@protoc_insertion_point(class_scope:harness.Input)
  })
_sym_db.RegisterMessage(Input)

Output = _reflection.GeneratedProtocolMessageType('Output', (_message.Message,), {
  'DESCRIPTOR' : _OUTPUT,
  '__module__' : 'harness.wire_pb2'
  # @@protoc_insertion_point(class_scope:harness.Output)
  })
_sym_db.RegisterMessage(Output)

Wire = _reflection.GeneratedProtocolMessageType('Wire', (_message.Message,), {
  'DESCRIPTOR' : _WIRE,
  '__module__' : 'harness.wire_pb2'
  # @@protoc_insertion_point(class_scope:harness.Wire)
  })
_sym_db.RegisterMessage(Wire)

Service = _reflection.GeneratedProtocolMessageType('Service', (_message.Message,), {

  'Resource' : _reflection.GeneratedProtocolMessageType('Resource', (_message.Message,), {
    'DESCRIPTOR' : _SERVICE_RESOURCE,
    '__module__' : 'harness.wire_pb2'
    # @@protoc_insertion_point(class_scope:harness.Service.Resource)
    })
  ,

  'Resources' : _reflection.GeneratedProtocolMessageType('Resources', (_message.Message,), {
    'DESCRIPTOR' : _SERVICE_RESOURCES,
    '__module__' : 'harness.wire_pb2'
    # @@protoc_insertion_point(class_scope:harness.Service.Resources)
    })
  ,

  'Container' : _reflection.GeneratedProtocolMessageType('Container', (_message.Message,), {
    'DESCRIPTOR' : _SERVICE_CONTAINER,
    '__module__' : 'harness.wire_pb2'
    # @@protoc_insertion_point(class_scope:harness.Service.Container)
    })
  ,
  'DESCRIPTOR' : _SERVICE,
  '__module__' : 'harness.wire_pb2'
  # @@protoc_insertion_point(class_scope:harness.Service)
  })
_sym_db.RegisterMessage(Service)
_sym_db.RegisterMessage(Service.Resource)
_sym_db.RegisterMessage(Service.Resources)
_sym_db.RegisterMessage(Service.Container)

wire.message_type = _WIRE
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(wire)
mark.message_type = _MARK
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(mark)
service.message_type = _SERVICE
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(service)

_INPUT.fields_by_name['type']._options = None
_OUTPUT.fields_by_name['type']._options = None
_WIRE.oneofs_by_name['type']._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: harness/postgres.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from harness import wire_pb2 as harness_dot_wire__pb2
from harness import net_pb2 as harness_dot_net__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='harness/postgres.proto',
  package='harness.postgres',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x16harness/postgres.proto\x12\x10harness.postgres\x1a\x12harness/wire.proto\x1a\x11harness/net.proto\"\x8d\x01\n\x04Pool\x12+\n\x07\x61\x64\x64ress\x18\x01 \x01(\x0b\x32\x13.harness.net.SocketB\x05\x92}\x02\x08\x00\x12\x10\n\x08username\x18\x02 \x01(\t\x12\x10\n\x08password\x18\x03 \x01(\t\x12\x10\n\x08\x64\x61tabase\x18\x04 \x01(\t\x12\x10\n\x08min_size\x18\x05 \x01(\r\x12\x10\n\x08max_size\x18\x06 \x01(\rb\x06proto3'
  ,
  dependencies=[harness_dot_wire__pb2.DESCRIPTOR,harness_dot_net__pb2.DESCRIPTOR,])




_POOL = _descriptor.Descriptor(
  name='Pool',
  full_name='harness.postgres.Pool',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='address', full_name='harness.postgres.Pool.address', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222}\002\010\000', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='username', full_name='harness.postgres.Pool.username', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='password', full_name='harness.postgres.Pool.password', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='database', full_name='harness.postgres.Pool.database', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='min_size', full_name='harness.postgres.Pool.min_size', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_size', full_name='harness.postgres.Pool.max_size', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=84,
  serialized_end=225,
)

_POOL.fields_by_name['address'].message_type = harness_dot_net__pb2._SOCKET
DESCRIPTOR.message_types_by_name['Pool'] = _POOL
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Pool = _reflection.GeneratedProtocolMessageType('Pool', (_message.Message,), {
  'DESCRIPTOR' : _POOL,
  '__module__' : 'harness.postgres_pb2'
  # @@protoc_insertion_point(class_scope:harness.postgres.Pool)
  })
_sym_db.RegisterMessage(Pool)


_POOL.fields_by_name['address']._options = None
# @@protoc_insertion_point(module_scope)



import logging

from prometheus_client import start_http_server

from opentelemetry import metrics
from opentelemetry.ext.prometheus import PrometheusMetricsExporter
from opentelemetry.sdk.metrics.export.controller import PushController

from .... import metrics_pb2
from ...base import Wire, WaitMixin


_log = logging.getLogger(__name__)


class PrometheusMetricsExporterWire(WaitMixin, Wire):
    """

    .. wire:: harness.wires.opentelemetry.ext.prometheus.PrometheusMetricsExporterWire
      :type: output
      :runtime: python
      :config: harness.metrics.Prometheus
      :requirements: opentelemetry-ext-prometheus==0.4a1

    """
    _config: metrics_pb2.Prometheus
    _controller: PushController

    def configure(self, value: metrics_pb2.Prometheus):
        assert isinstance(value, metrics_pb2.Prometheus), type(value)
        self._config = value

    async def __aenter__(self):
        meter = metrics.meter()
        exporter = PrometheusMetricsExporter(self._config.prefix)
        self._controller = PushController(meter, exporter, 5)

        start_http_server(self._config.bind.port, self._config.bind.host)
        _log.info('%s started: addr=%s:%d', self.__class__.__name__,
                  self._config.bind.host, self._config.bind.port)

    def close(self):
        super().close()
        if not self._controller.finished.is_set():
            self._controller.shutdown()



import yaml
from jsonpatch import JsonPatch
from json_merge_patch import merge


def load_config(config_content, merge_content=None, patch_content=None):
    config_data = yaml.safe_load(config_content)
    if config_data is None:
        config_data = {}
    if not isinstance(config_data, dict):
        raise SystemExit(f'Invalid configuration format: {type(config_data)!r}')

    if merge_content is not None:
        merge_data = yaml.safe_load(merge_content)
        config_data = merge(config_data, merge_data)

    if patch_content is not None:
        patch_data = yaml.safe_load(patch_content)
        json_patch = JsonPatch(patch_data)
        config_data = json_patch.apply(config_data)
    return config_data



from grpclib.plugin.main import Buffer

from ..config import ConfigSpec


def _render_wires(proto_file: str, spec: ConfigSpec):
    buf = Buffer()
    buf.add('# Generated by the Protocol Buffers compiler. DO NOT EDIT!')
    buf.add(f'# source: {proto_file}')
    buf.add(f'# plugin: {__name__}')

    if any(wire.optional for wire in spec.wires):
        buf.add('from typing import Optional')

    buf.add('from dataclasses import dataclass')
    buf.add('')

    imports = set()
    for wire in spec.wires:
        wire_type = wire.value.WhichOneof('type')
        if wire_type == 'input':
            value = wire.value.input.type
        elif wire_type == 'output':
            value = wire.value.output.type
        else:
            raise NotImplementedError(wire_type)
        module_name, _ = value.rsplit('.', 1)
        imports.add(module_name)
    for module_name in sorted(imports):
        buf.add(f'import {module_name}')
    buf.add('')
    buf.add('')

    buf.add('@dataclass')
    buf.add(f'class WiresIn:')
    with buf.indent():
        empty = True
        for wire in spec.wires:
            if wire.value.WhichOneof('type') == 'input':
                if wire.optional:
                    wire_type = f'Optional[{wire.value.input.type}]'
                else:
                    wire_type = wire.value.input.type
                buf.add(f'{wire.name}: {wire_type}')
                empty = False
        if empty:
            buf.add('pass')
    buf.add('')
    buf.add('')
    buf.add('@dataclass')
    buf.add(f'class WiresOut:')
    with buf.indent():
        empty = True
        for wire in spec.wires:
            if wire.value.WhichOneof('type') == 'output':
                buf.add(f'{wire.name}: {wire.value.output.type}')
                empty = False
        if empty:
            buf.add('pass')
    file_name = proto_file.replace('.proto', '_wires.py')
    return file_name, buf.content()


def _render_entrypoint(proto_file: str):
    buf = Buffer()
    buf.add('# Generated by the Protocol Buffers compiler. DO NOT EDIT!')
    buf.add(f'# source: {proto_file}')
    buf.add(f'# plugin: {__name__}')
    buf.add('import sys')
    buf.add('')
    buf.add('from harness.runtime import Runner')
    buf.add('')

    main_module = proto_file.replace('/', '.').replace('.proto', '')
    pb2_module = proto_file.replace('/', '.').replace('.proto', '_pb2')
    wires_module = proto_file.replace('/', '.').replace('.proto', '_wires')

    buf.add(f'import {main_module}')
    buf.add(f'import {pb2_module}')
    buf.add(f'import {wires_module}')
    buf.add('')
    buf.add('runner = Runner(')
    buf.add(f'    {pb2_module}.Configuration,')
    buf.add(f'    {wires_module}.WiresIn,')
    buf.add(f'    {wires_module}.WiresOut,')
    buf.add(')')
    buf.add('')
    buf.add("if __name__ == '__main__':")
    with buf.indent():
        buf.add(f'sys.exit(runner.run({main_module}.setup, sys.argv))')
    return 'entrypoint.py', buf.content()


def render(proto_file: str, spec: ConfigSpec):
    yield _render_wires(proto_file, spec)
    yield _render_entrypoint(proto_file)



import sys
import logging.handlers

from .. import logging_pb2

from .base import Wire


if hasattr(sys.stderr, 'isatty') and sys.stderr.isatty():
    try:
        import pygments
    except ImportError:
        _WITH_COLORS = False
    else:
        from pygments import highlight
        from pygments.console import ansiformat
        from pygments.lexers.python import PythonTracebackLexer
        from pygments.formatters.terminal import TerminalFormatter
        _WITH_COLORS = True
else:
    _WITH_COLORS = False


_LEVELS_MAP = {
    logging_pb2.Level.NOTSET: logging.NOTSET,
    logging_pb2.Level.DEBUG: logging.DEBUG,
    logging_pb2.Level.INFO: logging.INFO,
    logging_pb2.Level.WARNING: logging.WARNING,
    logging_pb2.Level.ERROR: logging.ERROR,
    logging_pb2.Level.CRITICAL: logging.CRITICAL,
}

_LEVEL_COLORS = {
    logging.DEBUG: 'brightblack',
    logging.INFO: 'green',
    logging.WARNING: 'yellow',
    logging.ERROR: 'red',
    logging.CRITICAL: '*red*',
}

_TEXT_COLORS = {
    logging.DEBUG: 'brightblack',
    logging.INFO: '',
    logging.WARNING: '',
    logging.ERROR: '',
    logging.CRITICAL: '',
}


def _wrap_text(level: int, value: str):
    return ansiformat(_TEXT_COLORS[level], value)


def _wrap_level(level, value: str):
    return ansiformat(_LEVEL_COLORS[level], value)


def _highlight(text):
    return highlight(text, lexer=PythonTracebackLexer(),
                     formatter=TerminalFormatter())


_FMT = '{asctime} {levelname} {name} {message}'


class _ColorFormatter(logging.Formatter):

    def format(self, record):
        record.asctime = self.formatTime(record)
        record.message = record.getMessage()
        message = _FMT.format(
            asctime=_wrap_text(record.levelno, record.asctime),
            levelname=_wrap_level(record.levelno, record.levelname),
            name=_wrap_text(record.levelno, record.name),
            message=_wrap_text(record.levelno, record.message),
        )
        if record.exc_info:
            exc_text = self.formatException(record.exc_info)
            if _WITH_COLORS:
                exc_text = _highlight(exc_text).rstrip('\n')
            message += '\n' + exc_text
        return message


class ConsoleWire(Wire):
    """

    .. wire:: harness.wires.logging.ConsoleWire
      :type: input
      :runtime: python
      :config: harness.logging.Console
      :requirements: pygments

    """
    _handler: logging.Handler

    def configure(self, value: logging_pb2.Console):
        logging.captureWarnings(True)

        self._handler = logging.StreamHandler()

        if _WITH_COLORS:
            self._handler.setFormatter(_ColorFormatter())
        else:
            self._handler.setFormatter(logging.Formatter(fmt=_FMT, style='{'))

        level = _LEVELS_MAP[value.level]
        self._handler.setLevel(level)

        logging.root.addHandler(self._handler)
        if level is not logging.NOTSET:
            if logging.root.level is not logging.NOTSET:
                level = min(level, logging.root.level)
            logging.root.setLevel(level)


class _SyslogHandler(logging.handlers.SysLogHandler):

    def __init__(self, *args, app_name, **kwargs):
        super().__init__(*args, **kwargs)
        self.ident = f'{app_name}: '


_FACILITY_MAP = {
    logging_pb2.Syslog.NOTSET: logging.handlers.SysLogHandler.LOG_USER,
    logging_pb2.Syslog.USER: logging.handlers.SysLogHandler.LOG_USER,
    logging_pb2.Syslog.LOCAL0: logging.handlers.SysLogHandler.LOG_LOCAL0,
    logging_pb2.Syslog.LOCAL1: logging.handlers.SysLogHandler.LOG_LOCAL1,
    logging_pb2.Syslog.LOCAL2: logging.handlers.SysLogHandler.LOG_LOCAL2,
    logging_pb2.Syslog.LOCAL3: logging.handlers.SysLogHandler.LOG_LOCAL3,
    logging_pb2.Syslog.LOCAL4: logging.handlers.SysLogHandler.LOG_LOCAL4,
    logging_pb2.Syslog.LOCAL5: logging.handlers.SysLogHandler.LOG_LOCAL5,
    logging_pb2.Syslog.LOCAL6: logging.handlers.SysLogHandler.LOG_LOCAL6,
    logging_pb2.Syslog.LOCAL7: logging.handlers.SysLogHandler.LOG_LOCAL7,
}


class SyslogWire(Wire):
    """

    .. wire:: harness.wires.logging.SyslogWire
      :type: input
      :runtime: python
      :config: harness.logging.Syslog
      :requirements:

    """
    _handler: logging

    def configure(self, value: logging_pb2.Syslog):
        logging.captureWarnings(True)

        self._handler = _SyslogHandler(
            app_name=value.app,
            address='/dev/log',
            facility=_FACILITY_MAP[value.facility],
        )

        level = _LEVELS_MAP[value.level]
        self._handler.setLevel(level)

        logging.root.addHandler(self._handler)
        if level is not logging.NOTSET:
            if logging.root.level is not logging.NOTSET:
                level = min(level, logging.root.level)
            logging.root.setLevel(level)



import logging
from typing import TYPE_CHECKING, List, Any
from contextvars import ContextVar

from opentelemetry.trace import get_tracer, SpanKind
from opentelemetry.propagators import extract

from grpclib.server import Server
from grpclib.events import listen, RecvRequest, SendTrailingMetadata
from grpclib.reflection.service import ServerReflection

from ... import grpc_pb2
from ..base import Wire

if TYPE_CHECKING:
    from typing_extensions import Protocol  # noqa

    class _Servable(Protocol):
        def __mapping__(self) -> Any: ...


_log = logging.getLogger(__name__)


def _metadata_getter(metadata, header_name: str) -> List[str]:
    return metadata.getall(header_name, [])


_server_span_ctx = ContextVar('server_span_ctx')


async def _recv_request(event: RecvRequest) -> None:
    tracer = get_tracer(__name__)
    parent_span = extract(_metadata_getter, event.metadata)
    span = tracer.start_span(
        event.method_name,
        parent_span,
        kind=SpanKind.SERVER,
        attributes={
            "component": "grpc",
            "grpc.method": event.method_name,
        },
    )
    span_ctx = tracer.use_span(span, True)
    span_ctx.__enter__()
    _server_span_ctx.set(span_ctx)


async def _send_trailing_metadata(event: SendTrailingMetadata) -> None:
    _server_span_ctx.get().__exit__(None, None, None)


class ServerWire(Wire):
    """

    .. wire:: harness.wires.grpclib.server.ServerWire
      :type: output
      :runtime: python
      :config: harness.grpc.Server
      :requirements:
        grpclib
        protobuf

    """
    _config: grpc_pb2.Server
    server: Server

    def __init__(self, handlers: List['_Servable']):
        self.handlers = handlers

    def configure(self, value: grpc_pb2.Server):
        assert isinstance(value, grpc_pb2.Server), type(value)
        self._config = value

        handlers = list(self.handlers)
        services = ServerReflection.extend(handlers)
        self.server = Server(services)
        listen(self.server, RecvRequest, _recv_request)
        listen(self.server, SendTrailingMetadata, _send_trailing_metadata)

    async def __aenter__(self):
        await self.server.start(self._config.bind.host, self._config.bind.port)
        _log.info('%s started: addr=%s:%d', self.__class__.__name__,
                  self._config.bind.host, self._config.bind.port)

    def close(self):
        self.server.close()

    async def wait_closed(self):
        await self.server.wait_closed()



def enable_metrics():
    from opentelemetry.metrics import set_preferred_meter_implementation
    from opentelemetry.sdk.metrics import Meter
    set_preferred_meter_implementation(lambda _: Meter())


def enable_tracing():
    from opentelemetry.trace import set_preferred_tracer_source_implementation
    from opentelemetry.sdk.trace import TracerSource
    set_preferred_tracer_source_implementation(lambda _: TracerSource())



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: harness/tracing.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from harness import net_pb2 as harness_dot_net__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='harness/tracing.proto',
  package='harness.tracing',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x15harness/tracing.proto\x12\x0fharness.tracing\x1a\x11harness/net.proto\x1a\x17validate/validate.proto\"M\n\x06Jaeger\x12\x1d\n\x0cservice_name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12$\n\x07\x61\x64\x64ress\x18\x02 \x01(\x0b\x32\x13.harness.net.Socketb\x06proto3'
  ,
  dependencies=[harness_dot_net__pb2.DESCRIPTOR,validate_dot_validate__pb2.DESCRIPTOR,])




_JAEGER = _descriptor.Descriptor(
  name='Jaeger',
  full_name='harness.tracing.Jaeger',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='service_name', full_name='harness.tracing.Jaeger.service_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\004r\002\020\001', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='address', full_name='harness.tracing.Jaeger.address', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=86,
  serialized_end=163,
)

_JAEGER.fields_by_name['address'].message_type = harness_dot_net__pb2._SOCKET
DESCRIPTOR.message_types_by_name['Jaeger'] = _JAEGER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Jaeger = _reflection.GeneratedProtocolMessageType('Jaeger', (_message.Message,), {
  'DESCRIPTOR' : _JAEGER,
  '__module__' : 'harness.tracing_pb2'
  # @@protoc_insertion_point(class_scope:harness.tracing.Jaeger)
  })
_sym_db.RegisterMessage(Jaeger)


_JAEGER.fields_by_name['service_name']._options = None
# @@protoc_insertion_point(module_scope)



from apscheduler.schedulers.base import BaseScheduler

from ..base import Wire, WaitMixin


class SchedulerWire(WaitMixin, Wire):
    """

    .. wire:: harness.wires.apscheduler.SchedulerWire
      :type: output
      :runtime: python
      :config: google.protobuf.Empty
      :requirements: apscheduler

    """
    def __init__(self, scheduler: 'BaseScheduler'):
        self._scheduler = scheduler

    async def __aenter__(self):
        self._scheduler.start()

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        self._scheduler.shutdown()



import sys
from argparse import FileType
from contextlib import closing

from google.protobuf.json_format import ParseDict, ParseError
from google.protobuf.message_factory import GetMessages

from ..runtime._utils import load_config
from ..runtime._validate import validate, ValidationError

from .utils import get_configuration, load_descriptor_set


def check(args):
    with closing(args.config):
        config_content = args.config.read()

    if args.merge:
        with closing(args.merge):
            merge_content = args.merge.read()
    else:
        merge_content = None

    if args.patch:
        with closing(args.patch):
            patch_content = args.patch.read()
    else:
        patch_content = None

    descriptor_set = load_descriptor_set(args.proto, args.proto_path)
    file_descriptor, = (f for f in descriptor_set.file
                        if args.proto.endswith(f.name))

    config_descriptor = get_configuration(file_descriptor)
    if config_descriptor is None:
        raise Exception(f'Configuration message not found in the {args.proto}')

    config_data = load_config(
        config_content=config_content,
        merge_content=merge_content,
        patch_content=patch_content,
    )
    message_classes = GetMessages(descriptor_set.file)
    config_full_name = '.'.join(filter(None, (
        file_descriptor.package,
        config_descriptor.name,
    )))
    config_cls = message_classes[config_full_name]
    config = config_cls()
    try:
        ParseDict(config_data, config)
    except ParseError as err:
        print(f'Parse error: {err}')
        sys.exit(1)
    try:
        validate(config)
    except ValidationError as err:
        print(f'Validation error: {err}')
        sys.exit(1)


def add_commands(subparsers):
    check_parser = subparsers.add_parser('check')
    check_parser.add_argument('-I', '--proto-path', action='append')
    check_parser.add_argument('proto')
    check_parser.add_argument('config', type=FileType(encoding='utf-8'))
    check_parser.add_argument('--merge', type=FileType(encoding='utf-8'),
                              default=None)
    check_parser.add_argument('--patch', type=FileType(encoding='utf-8'),
                              default=None)
    check_parser.set_defaults(func=check)



import tempfile
from pathlib import Path

import pkg_resources
from grpc_tools import protoc
from google.protobuf.descriptor_pb2 import FileDescriptorSet


def load_descriptor_set(proto_file, proto_path=None):
    wkt_protos = pkg_resources.resource_filename('grpc_tools', '_proto')
    validate_protos = str(Path(
        pkg_resources.resource_filename('validate', 'validate.proto'))
        .parent.parent
    )
    harness_protos = str(Path(__file__).parent.parent.parent.absolute())
    with tempfile.NamedTemporaryFile() as f:
        args = [
            'grpc_tools.protoc',
            '--include_imports',
            f'--proto_path={wkt_protos}',
            f'--proto_path={validate_protos}',
            f'--proto_path={harness_protos}',
            f'--proto_path={Path(proto_file).parent}',
            f'--descriptor_set_out={f.name}',
        ]
        if proto_path:
            args.extend(f'--proto_path={p}' for p in proto_path)
        args.append(proto_file)
        result = protoc.main(args)
        if result != 0:
            raise Exception('Failed to call protoc')
        content = f.read()
    return FileDescriptorSet.FromString(content)


def get_configuration(file_descriptor):
    for message_type in file_descriptor.message_type:
        if message_type.name == 'Configuration':
            return message_type



from contextvars import ContextVar

from opentelemetry.trace import get_tracer, SpanKind
from opentelemetry.propagators import inject

from grpclib.client import Channel
from grpclib.events import listen, SendRequest, RecvTrailingMetadata

from harness import grpc_pb2
from harness.wires.base import Wire


_client_span = ContextVar('client_span')


async def _send_request(event: SendRequest) -> None:
    tracer = get_tracer(__name__)
    span = tracer.start_span(
        event.method_name,
        kind=SpanKind.CLIENT,
        attributes={
            "component": "grpc",
            "grpc.method": event.method_name,
        },
    )
    _client_span.set(span)
    inject(tracer, type(event.metadata).__setitem__, event.metadata)


async def _recv_trailing_metadata(event: RecvTrailingMetadata) -> None:
    _client_span.get().end()


class ChannelWire(Wire):
    """

    .. wire:: harness.wires.grpclib.client.ChannelWire
      :type: input
      :runtime: python
      :config: harness.grpc.Channel
      :requirements:
        grpclib
        protobuf

    """
    channel: Channel

    def configure(self, value: grpc_pb2.Channel):
        assert isinstance(value, grpc_pb2.Channel), type(value)

        self.channel = Channel(value.address.host, value.address.port)
        listen(self.channel, SendRequest, _send_request)
        listen(self.channel, RecvTrailingMetadata, _recv_trailing_metadata)

    def close(self):
        self.channel.close()



import logging

from prometheus_client import start_http_server

from .. import http_pb2

from .base import Wire, WaitMixin


_log = logging.getLogger(__name__)


class ServerWire(WaitMixin, Wire):
    """

    .. wire:: harness.wires.prometheus.ServerWire
      :type: output
      :runtime: python
      :config: harness.http.Server
      :requirements: prometheus_client

    """
    _config: http_pb2.Server

    def configure(self, value: http_pb2.Server):
        assert isinstance(value, http_pb2.Server), type(value)
        self._config = value

    async def __aenter__(self):
        start_http_server(self._config.bind.port, self._config.bind.host)
        _log.info('%s started: addr=%s:%d', self.__class__.__name__,
                  self._config.bind.host, self._config.bind.port)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: harness/metrics.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from harness import net_pb2 as harness_dot_net__pb2
from harness import wire_pb2 as harness_dot_wire__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='harness/metrics.proto',
  package='harness.metrics',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x15harness/metrics.proto\x12\x0fharness.metrics\x1a\x11harness/net.proto\x1a\x12harness/wire.proto\"F\n\nPrometheus\x12\x0e\n\x06prefix\x18\x01 \x01(\t\x12(\n\x04\x62ind\x18\x02 \x01(\x0b\x32\x13.harness.net.SocketB\x05\x92}\x02\x08\x01\x62\x06proto3'
  ,
  dependencies=[harness_dot_net__pb2.DESCRIPTOR,harness_dot_wire__pb2.DESCRIPTOR,])




_PROMETHEUS = _descriptor.Descriptor(
  name='Prometheus',
  full_name='harness.metrics.Prometheus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='prefix', full_name='harness.metrics.Prometheus.prefix', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bind', full_name='harness.metrics.Prometheus.bind', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222}\002\010\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=81,
  serialized_end=151,
)

_PROMETHEUS.fields_by_name['bind'].message_type = harness_dot_net__pb2._SOCKET
DESCRIPTOR.message_types_by_name['Prometheus'] = _PROMETHEUS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Prometheus = _reflection.GeneratedProtocolMessageType('Prometheus', (_message.Message,), {
  'DESCRIPTOR' : _PROMETHEUS,
  '__module__' : 'harness.metrics_pb2'
  # @@protoc_insertion_point(class_scope:harness.metrics.Prometheus)
  })
_sym_db.RegisterMessage(Prometheus)


_PROMETHEUS.fields_by_name['bind']._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: harness/grpc.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from harness import net_pb2 as harness_dot_net__pb2
from harness import wire_pb2 as harness_dot_wire__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='harness/grpc.proto',
  package='harness.grpc',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x12harness/grpc.proto\x12\x0charness.grpc\x1a\x11harness/net.proto\x1a\x12harness/wire.proto\"6\n\x07\x43hannel\x12+\n\x07\x61\x64\x64ress\x18\x01 \x01(\x0b\x32\x13.harness.net.SocketB\x05\x92}\x02\x08\x02\"2\n\x06Server\x12(\n\x04\x62ind\x18\x01 \x01(\x0b\x32\x13.harness.net.SocketB\x05\x92}\x02\x08\x02\x62\x06proto3'
  ,
  dependencies=[harness_dot_net__pb2.DESCRIPTOR,harness_dot_wire__pb2.DESCRIPTOR,])




_CHANNEL = _descriptor.Descriptor(
  name='Channel',
  full_name='harness.grpc.Channel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='address', full_name='harness.grpc.Channel.address', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222}\002\010\002', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=75,
  serialized_end=129,
)


_SERVER = _descriptor.Descriptor(
  name='Server',
  full_name='harness.grpc.Server',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='bind', full_name='harness.grpc.Server.bind', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222}\002\010\002', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=131,
  serialized_end=181,
)

_CHANNEL.fields_by_name['address'].message_type = harness_dot_net__pb2._SOCKET
_SERVER.fields_by_name['bind'].message_type = harness_dot_net__pb2._SOCKET
DESCRIPTOR.message_types_by_name['Channel'] = _CHANNEL
DESCRIPTOR.message_types_by_name['Server'] = _SERVER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Channel = _reflection.GeneratedProtocolMessageType('Channel', (_message.Message,), {
  'DESCRIPTOR' : _CHANNEL,
  '__module__' : 'harness.grpc_pb2'
  # @@protoc_insertion_point(class_scope:harness.grpc.Channel)
  })
_sym_db.RegisterMessage(Channel)

Server = _reflection.GeneratedProtocolMessageType('Server', (_message.Message,), {
  'DESCRIPTOR' : _SERVER,
  '__module__' : 'harness.grpc_pb2'
  # @@protoc_insertion_point(class_scope:harness.grpc.Server)
  })
_sym_db.RegisterMessage(Server)


_CHANNEL.fields_by_name['address']._options = None
_SERVER.fields_by_name['bind']._options = None
# @@protoc_insertion_point(module_scope)



import sys
import argparse


def main():
    parser = argparse.ArgumentParser()
    subparser = parser.add_subparsers()

    from . import kubegen, check, protopath

    kubegen.add_commands(subparser)
    check.add_commands(subparser)
    protopath.add_commands(subparser)

    args = parser.parse_args()
    if 'func' in args:
        sys.exit(args.func(args) or 0)
    else:
        parser.print_help()



import logging

from opentelemetry.trace import tracer_source
from opentelemetry.sdk.trace import TracerSource
from opentelemetry.ext.jaeger import JaegerSpanExporter
from opentelemetry.sdk.trace.export import BatchExportSpanProcessor

from .... import tracing_pb2
from ...base import Wire


_log = logging.getLogger(__name__)


class JaegerSpanExporterWire(Wire):
    """

    .. wire:: harness.wires.opentelemetry.ext.jaeger.JaegerSpanExporterWire
      :type: input
      :runtime: python
      :config: harness.tracing.Jaeger
      :requirements: opentelemetry-ext-jaeger==0.4a1

    """
    _config: tracing_pb2.Jaeger

    def configure(self, value: tracing_pb2.Jaeger):
        assert isinstance(value, tracing_pb2.Jaeger), type(value)
        self._config = value

    async def __aenter__(self):
        exporter = JaegerSpanExporter(
            service_name=self._config.service_name,
            agent_host_name=self._config.address.host,
            agent_port=self._config.address.port,
        )
        span_processor = BatchExportSpanProcessor(exporter)
        source: TracerSource = tracer_source()
        source.add_span_processor(span_processor)
        _log.info('%s started: service_name=%s; addr=%s:%d',
                  self.__class__.__name__, self._config.service_name,
                  self._config.address.host, self._config.address.port)



import re
import ipaddress
from abc import ABC, abstractmethod
from typing import Union, List
from decimal import Decimal
from collections import Counter
from urllib.parse import urlparse
from email.headerregistry import AddressHeader

from grpclib.plugin.main import Buffer
from google.protobuf.message import Message
from google.protobuf.descriptor import Descriptor, FieldDescriptor
from google.protobuf.duration_pb2 import Duration
from google.protobuf.timestamp_pb2 import Timestamp

from validate import validate_pb2


class ValidationError(ValueError):
    pass


class Format:
    # Based on Django 3.0.2: django.core.validators.URLValidator
    _scheme = r'^(?:[a-z0-9\.\-\+]*)://'
    _user = r'(?:[^\s:@/]+(?::[^\s:@/]*)?@)?'
    _ipv4 = r'(?:25[0-5]|2[0-4]\d|[0-1]?\d?\d)(?:\.(?:25[0-5]|2[0-4]\d|[0-1]?\d?\d)){3}'
    _ipv6 = r'\[[0-9a-f:\.]+\]'  # (simple regex, validated later)
    _host = (
        r'('
        # hostname
        r'[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?'
        # domain
        r'(?:\.(?!-)[a-z0-9-]{1,63}(?<!-))*'
        # tld
        r'\.(?!-)(?:[a-z-]{2,63})(?<!-)\.?'
        # special
        r'|localhost)'
    )
    _addr = r'(?:' + _ipv4 + '|' + _ipv6 + '|' + _host + ')'
    _port = r'(?::\d{2,5})?'
    _resource = r'(?:[/?#][^\s]*)?'
    _uri = _scheme + _user + _addr + _port + _resource
    _uri_ref = r'(?:' + _scheme + _user + _addr + _port + ')?' + _resource

    _host_re = re.compile('^' + _host + '$', re.IGNORECASE)
    _uri_re = re.compile('^' + _uri + '$', re.IGNORECASE)
    _uri_ref_re = re.compile('^' + _uri_ref + '$', re.IGNORECASE)

    _uuid_re = re.compile(
        '^[0-9a-f]{8}'
        '-[0-9a-f]{4}'
        '-[0-9a-f]{4}'
        '-[0-9a-f]{4}'
        '-[0-9a-f]{12}$',
        re.IGNORECASE,
    )

    def check_email(self, field_name, value: str):
        result = {}
        AddressHeader.parse(value, result)
        defects = result['defects']
        if defects:
            errors = '\n'.join(f' - {str(defect)}' for defect in defects)
            raise ValidationError(f'{field_name} contains invalid email address:\n{errors}')

        groups = result['groups']
        if len(groups) > 1 or len(groups[0].addresses) > 1:
            raise ValidationError(f'{field_name} contains more than one email address')

    def check_hostname(self, field_name, value: str):
        if self._host_re.match(value) is None:
            raise ValidationError(f'{field_name} contains invalid hostname')

    def check_ip(self, field_name, value: Union[str, bytes]):
        try:
            ipaddress.ip_address(value)
        except ValueError:
            raise ValidationError(f'{field_name} contains invalid IP address') from None

    def check_ipv4(self, field_name, value: Union[str, bytes]):
        try:
            ipaddress.IPv4Address(value)
        except ValueError:
            raise ValidationError(f'{field_name} contains invalid IPv4 address') from None

    def check_ipv6(self, field_name, value: Union[str, bytes]):
        try:
            ipaddress.IPv6Address(value)
        except ValueError:
            raise ValidationError(f'{field_name} contains invalid IPv6 address') from None

    def check_uri(self, field_name, value: str):
        if not self._uri_re.match(value):
            raise ValidationError(f'{field_name} contains invalid URI')
        url = urlparse(value)
        assert url.hostname, url
        self.check_address(field_name, url.hostname)

    def check_uri_ref(self, field_name, value: str):
        if not self._uri_ref_re.match(value):
            raise ValidationError(f'{field_name} contains invalid URI-reference')
        url = urlparse(value)
        if url.hostname:
            self.check_address(field_name, url.hostname)

    def check_address(self, field_name, value: str):
        if self._host_re.match(value) is None:
            try:
                ipaddress.ip_address(value)
                return
            except ValueError:
                pass
        else:
            return
        raise ValidationError(f'{field_name} contains invalid address')

    def check_uuid(self, field_name, value: str):
        if self._uuid_re.match(value) is None:
            raise ValidationError(f'{field_name} contains invalid UUID') from None


def dec(value: str):
    return Decimal(value)


def dec_repr(value: Union[Timestamp, Duration]):
    return f'{value.seconds}.{value.nanos:09d}'


def sec(value: Union[Timestamp, Duration]):
    return Decimal(dec_repr(value))


def now():
    ts = Timestamp()
    ts.GetCurrentTime()
    return sec(ts)


def err_gen(buf, message, **ctx):
    message_repr = repr(message)
    if ctx:
        ctx_items = ', '.join(
            '{}={}'.format(key, val) for key, val in ctx.items()
        )
        message_repr += f'.format({ctx_items})'
    buf.add(f'raise {ValidationError.__name__}({message_repr})')


class FieldRulesBase:
    buf: Buffer
    field: FieldDescriptor
    code_path: List[str]
    proto_path: List[str]

    def field_value(self):
        return '.'.join(self.code_path)

    def proto_name(self):
        return ''.join(self.proto_path)

    def rule_value(self, value):
        return repr(value)

    def rule_value_repr(self, value):
        return repr(value)


class FieldRules(FieldRulesBase, ABC):

    def __init__(self, buf: Buffer, field: FieldDescriptor, code_path: List[str], proto_path: List[str]) -> None:
        self.buf = buf
        self.field = field
        self.code_path = code_path
        self.proto_path = proto_path

    @property
    @abstractmethod
    def rule_descriptor(self) -> Descriptor:
        pass

    def dispatch(self, rules):
        for field in self.rule_descriptor.fields:
            if field.label == FieldDescriptor.LABEL_REPEATED:
                rule_value = getattr(rules, field.name)
                if not rule_value:
                    continue
            else:
                if not rules.HasField(field.name):
                    continue
                rule_value = getattr(rules, field.name)
            try:
                visit_fn = getattr(self, f'visit_{field.name}')
            except AttributeError:
                raise NotImplementedError(field.full_name)
            visit_fn(rule_value)


class ConstRulesMixin(FieldRulesBase):

    def visit_const(self, value):
        self.buf.add(f'if {self.field_value()} != {self.rule_value(value)}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} not equal to {self.rule_value_repr(value)}')


class InRulesMixin(FieldRulesBase):

    def _set(self, value):
        return '{{' + ', '.join([self.rule_value(i) for i in value]) + '}}'

    def _set_repr(self, value):
        return '{{' + ', '.join([self.rule_value_repr(i) for i in value]) + '}}'

    def visit_in(self, value):
        value_set = self._set(value)
        value_set_repr = self._set_repr(value)
        self.buf.add(f'if {self.field_value()} not in {value_set}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} not in {value_set_repr}')

    def visit_not_in(self, value):
        value_set = self._set(value)
        value_set_repr = self._set_repr(value)
        self.buf.add(f'if {self.field_value()} in {value_set}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} in {value_set_repr}')


class ComparatorRulesMixin(FieldRulesBase):

    def visit_lt(self, value):
        self.buf.add(f'if not {self.field_value()} < {self.rule_value(value)}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} is not lesser than {self.rule_value_repr(value)}')

    def visit_lte(self, value):
        self.buf.add(f'if not {self.field_value()} <= {self.rule_value(value)}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} is not lesser than or equal to {self.rule_value_repr(value)}')

    def visit_gt(self, value):
        self.buf.add(f'if not {self.field_value()} > {self.rule_value(value)}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} is not greater than {self.rule_value_repr(value)}')

    def visit_gte(self, value):
        self.buf.add(f'if not {self.field_value()} >= {self.rule_value(value)}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} is not greater than or equal to {self.rule_value_repr(value)}')


class SizableRulesMixin(FieldRulesBase):

    def visit_len(self, value):
        self.buf.add(f'if len({self.field_value()}) != {value}:')
        with self.buf.indent():
            err_gen(self.buf,
                    f'{self.proto_name()} length does not equal {value}')

    def visit_min_len(self, value):
        self.buf.add(f'if len({self.field_value()}) < {value}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} length is less than {value}')

    def visit_max_len(self, value):
        self.buf.add(f'if len({self.field_value()}) > {value}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} length is more than {value}')


class FloatRules(ConstRulesMixin, InRulesMixin, ComparatorRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.FloatRules.DESCRIPTOR


class DoubleRules(ConstRulesMixin, InRulesMixin, ComparatorRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.DoubleRules.DESCRIPTOR


class Int32Rules(ConstRulesMixin, InRulesMixin, ComparatorRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.Int32Rules.DESCRIPTOR


class Int64Rules(ConstRulesMixin, InRulesMixin, ComparatorRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.Int64Rules.DESCRIPTOR


class UInt32Rules(ConstRulesMixin, InRulesMixin, ComparatorRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.UInt32Rules.DESCRIPTOR


class UInt64Rules(ConstRulesMixin, InRulesMixin, ComparatorRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.UInt64Rules.DESCRIPTOR


class SInt32Rules(ConstRulesMixin, InRulesMixin, ComparatorRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.SInt32Rules.DESCRIPTOR


class SInt64Rules(ConstRulesMixin, InRulesMixin, ComparatorRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.SInt64Rules.DESCRIPTOR


class Fixed32Rules(ConstRulesMixin, InRulesMixin, ComparatorRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.Fixed32Rules.DESCRIPTOR


class Fixed64Rules(ConstRulesMixin, InRulesMixin, ComparatorRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.Fixed64Rules.DESCRIPTOR


class SFixed32Rules(ConstRulesMixin, InRulesMixin, ComparatorRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.SFixed32Rules.DESCRIPTOR


class SFixed64Rules(ConstRulesMixin, InRulesMixin, ComparatorRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.SFixed64Rules.DESCRIPTOR


class BoolRules(ConstRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.BoolRules.DESCRIPTOR


class CommonStringRules(FieldRulesBase):

    def visit_prefix(self, value):
        self.buf.add(f'if not {self.field_value()}.startswith({value}):')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} does not start with prefix {value}')

    def visit_suffix(self, value):
        self.buf.add(f'if not {self.field_value()}.endswith({value}):')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} does not end with suffix {value}')

    def visit_contains(self, value):
        self.buf.add(f'if {value} not in {self.field_value()}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} does not contain {value}')

    def visit_not_contains(self, value):
        self.buf.add(f'if {value} in {self.field_value()}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} contains {value}')


class StringRules(CommonStringRules, ConstRulesMixin, InRulesMixin, SizableRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.StringRules.DESCRIPTOR

    def visit_len_bytes(self, value):
        self.buf.add(f'if len({self.field_value()}.encode("utf-8")) != {value}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} bytes count does not equal {value}')

    def visit_min_bytes(self, value):
        self.buf.add(f'if len({self.field_value()}.encode("utf-8")) < {value}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} bytes count is less than {value}')

    def visit_max_bytes(self, value):
        self.buf.add(f'if len({self.field_value()}.encode("utf-8")) > {value}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} bytes count is more than {value}')

    def visit_pattern(self, value):
        self.buf.add(f'if re.search({value}, {self.field_value()}) is None:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} does not match pattern {value}')

    def visit_email(self, _):
        self.buf.add(f'fmt.check_email({self.proto_name()!r}, {self.field_value()})')

    def visit_hostname(self, _):
        self.buf.add(f'fmt.check_hostname({self.proto_name()!r}, {self.field_value()})')

    def visit_ip(self, _):
        self.buf.add(f'fmt.check_ip({self.proto_name()!r}, {self.field_value()})')

    def visit_ipv4(self, _):
        self.buf.add(f'fmt.check_ipv4({self.proto_name()!r}, {self.field_value()})')

    def visit_ipv6(self, _):
        self.buf.add(f'fmt.check_ipv6({self.proto_name()!r}, {self.field_value()})')

    def visit_uri(self, _):
        self.buf.add(f'fmt.check_uri({self.proto_name()!r}, {self.field_value()})')

    def visit_uri_ref(self, _):
        self.buf.add(f'fmt.check_uri_ref({self.proto_name()!r}, {self.field_value()})')

    def visit_address(self, _):
        self.buf.add(f'fmt.check_address({self.proto_name()!r}, {self.field_value()})')

    def visit_uuid(self, _):
        self.buf.add(f'fmt.check_uuid({self.proto_name()!r}, {self.field_value()})')


class BytesRules(CommonStringRules, ConstRulesMixin, InRulesMixin, SizableRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.BytesRules.DESCRIPTOR

    def visit_pattern(self, value):
        try:
            value_bytes = value.encode('ascii')
        except UnicodeEncodeError:
            err_gen(self.buf, f'{self.proto_name()} has invalid validation pattern {value}')
        else:
            self.buf.add(f'if re.search({value_bytes!r}, {self.field_value()}) is None:')
            with self.buf.indent():
                err_gen(self.buf, f'{self.proto_name()} does not match pattern {value_bytes!r}')

    def visit_ip(self, _):
        self.buf.add(f'fmt.check_ip({self.proto_name()!r}, {self.field_value()})')

    def visit_ipv4(self, _):
        self.buf.add(f'fmt.check_ipv4({self.proto_name()!r}, {self.field_value()})')

    def visit_ipv6(self, _):
        self.buf.add(f'fmt.check_ipv6({self.proto_name()!r}, {self.field_value()})')


class EnumRules(ConstRulesMixin, InRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.EnumRules.DESCRIPTOR

    def visit_defined_only(self, _):
        ids = [e.number for e in self.field.enum_type.values]
        ids_repr = '{{' + ', '.join(map(repr, ids)) + '}}'
        self.buf.add(f'if {self.field_value()} not in {ids_repr}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} is not defined')


class RepeatedRules(FieldRules):
    rule_descriptor = validate_pb2.RepeatedRules.DESCRIPTOR

    def visit_min_items(self, value: int):
        self.buf.add(f'if len({self.field_value()}) < {value}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} items count is less than {value}')

    def visit_max_items(self, value: int):
        self.buf.add(f'if len({self.field_value()}) > {value}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} items count is more than {value}')

    def visit_unique(self, _):
        self.buf.add(f'if len(set({self.field_value()})) < len({self.field_value()}):')
        with self.buf.indent():
            repeated = f'[i for i, count in Counter({self.field_value()}).items() if count > 1]'
            err_gen(self.buf, f'{self.proto_name()} must contain unique items; repeated items: {{{{repeated}}}}', repeated=repeated)

    def visit_items(self, value: validate_pb2.FieldRules):
        self.buf.add(f'for item in {self.field_value()}:')
        with self.buf.indent():
            proto_path = self.proto_path + ['[]']
            dispatch_field(self.buf, self.field, ['item'], proto_path, value)


class MapRules(FieldRules):
    rule_descriptor = validate_pb2.MapRules.DESCRIPTOR

    def visit_min_pairs(self, value: int):
        self.buf.add(f'if len({self.field_value()}) < {value}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} needs to contain at least {value} items')

    def visit_max_pairs(self, value: int):
        self.buf.add(f'if len({self.field_value()}) > {value}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} can contain at most {value} items')

    def visit_no_sparse(self, _):
        # no_sparse validation is not implemented because protobuf maps
        # cannot be sparse in Python
        pass

    def visit_keys(self, value: validate_pb2.FieldRules):
        self.buf.add(f'for key in {self.field_value()}.keys():')
        with self.buf.indent():
            proto_path = self.proto_path + ['<key>']
            dispatch_field(self.buf, self.field, ['key'], proto_path, value)

    def visit_values(self, value: validate_pb2.FieldRules):
        self.buf.add(f'for value in {self.field_value()}.values():')
        with self.buf.indent():
            proto_path = self.proto_path + ['<value>']
            dispatch_field(self.buf, self.field, ['value'], proto_path, value)


class MessageRulesMixin(FieldRulesBase):

    def visit_required(self, _):
        container = '.'.join(self.code_path[:-1])
        field_name = self.code_path[-1]
        self.buf.add(f'if not {container}.HasField({field_name!r}):')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} is required')


class MessageRules(MessageRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.MessageRules.DESCRIPTOR

    def visit_skip(self, _):
        # This option is handled explicitly outside
        pass


class AnyRules(MessageRulesMixin, InRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.AnyRules.DESCRIPTOR

    def field_value(self):
        return f'{super().field_value()}.type_url'

    def proto_name(self):
        return f'{super().proto_name()}.type_url'

    def visit_in(self, value):
        self.buf.add('print(p.field.type_url)')
        value_set = self._set(value)
        value_set_repr = self._set_repr(value)
        self.buf.add(f'if {self.field_value()} not in {value_set}:')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} not in {value_set_repr}')


class TimeFormatMixin(FieldRulesBase):

    def field_value(self):
        return f'sec({super().field_value()})'

    def rule_value(self, value):
        return f'dec("{dec_repr(value)}")'

    def rule_value_repr(self, value):
        return value.ToJsonString()


class DurationRules(TimeFormatMixin, MessageRulesMixin, ConstRulesMixin, ComparatorRulesMixin, InRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.DurationRules.DESCRIPTOR


class TimestampRules(TimeFormatMixin, MessageRulesMixin, ConstRulesMixin, ComparatorRulesMixin, FieldRules):
    rule_descriptor = validate_pb2.TimestampRules.DESCRIPTOR

    lt_now: bool
    gt_now: bool

    def visit_lt_now(self, _):
        # this rule is used only in conjunction with "within" rule, and handled
        # in visit_within method
        pass

    def visit_gt_now(self, _):
        # this rule is used only in conjunction with "within" rule, and handled
        # in visit_within method
        pass

    def visit_within(self, value):
        if self.lt_now:
            self.visit_within_past(value)
        elif self.gt_now:
            self.visit_within_future(value)
        else:
            self.buf.add(f'if not abs({self.field_value()} - now()) < {self.rule_value(value)}:')
            with self.buf.indent():
                err_gen(self.buf, f'{self.proto_name()} is not within {self.rule_value_repr(value)} from now')

    def visit_within_past(self, value):
        self.buf.add(f'if not (0 < now() - {self.field_value()} < {self.rule_value(value)}):')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} is not within {self.rule_value_repr(value)} in the past')

    def visit_within_future(self, value):
        self.buf.add(f'if not (0 < {self.field_value()} - now() < {self.rule_value(value)}):')
        with self.buf.indent():
            err_gen(self.buf, f'{self.proto_name()} is not within {self.rule_value_repr(value)} in the future')

    def dispatch(self, rules):
        self.lt_now = rules.lt_now
        self.gt_now = rules.gt_now
        super().dispatch(rules)


FIELD_RULE_TYPES = {r.rule_descriptor.full_name: r for r in [
    FloatRules,
    DoubleRules,
    Int32Rules,
    Int64Rules,
    UInt32Rules,
    UInt64Rules,
    SInt32Rules,
    SInt64Rules,
    Fixed32Rules,
    Fixed64Rules,
    SFixed32Rules,
    SFixed64Rules,
    BoolRules,
    StringRules,
    BytesRules,
    EnumRules,
    RepeatedRules,
    MapRules,
    AnyRules,
    DurationRules,
    TimestampRules,
]}


def dispatch_field(buf, field, code_path, proto_path, opt_value):
    if opt_value.HasField('message'):
        assert field.message_type
        MessageRules(buf, field, code_path, proto_path).dispatch(opt_value.message)
    oneof_type = opt_value.WhichOneof('type')
    if oneof_type:
        rule_value = getattr(opt_value, oneof_type)
        generator = FIELD_RULE_TYPES[rule_value.DESCRIPTOR.full_name]
        generator(buf, field, code_path, proto_path).dispatch(rule_value)


def field_gen(buf, field, code_path, proto_path):
    for opt, opt_value in field.GetOptions().ListFields():
        if opt.full_name == "validate.rules":
            dispatch_field(buf, field, code_path, proto_path, opt_value)
            if opt_value.HasField('message'):
                assert field.message_type
                if opt_value.message.skip:
                    return

    if field.message_type:
        for opt, opt_value in field.message_type.GetOptions().ListFields():
            if opt.full_name == 'google.protobuf.MessageOptions.map_entry' and opt_value is True:
                return
        field_value = '.'.join(code_path)
        if field.label == FieldDescriptor.LABEL_REPEATED:
            buf.add(f'for item in {field_value}:')
            with buf.indent():
                buf.add(f'validate(item)')
        else:
            outer = '.'.join(code_path[:-1])
            inner = code_path[-1]
            buf.add(f'if {outer}.HasField("{inner}"):')
            with buf.indent():
                buf.add(f'validate({field_value})')


def file_gen(message):
    for opt, opt_value in message.DESCRIPTOR.GetOptions().ListFields():
        if opt.full_name == "validate.disabled" and opt_value:
            return

    buf = Buffer()
    buf.add('def _validate(p):')
    pos = len(buf._lines)
    with buf.indent():
        for oneof in message.DESCRIPTOR.oneofs:
            buf.add(f"__{oneof.name} = p.WhichOneof('{oneof.name}')")
            for i, field in enumerate(oneof.fields):
                ctrl = 'elif' if i else 'if'
                buf.add(f"{ctrl} __{oneof.name} == '{field.name}':")
                with buf.indent():
                    inner_pos = len(buf._lines)
                    field_gen(buf, field, ['p', field.name], [field.name])
                    if len(buf._lines) == inner_pos:
                        buf.add('pass')
            for opt, opt_value in oneof.GetOptions().ListFields():
                if opt.name == 'required' and opt_value:
                    buf.add(f"else:")
                    with buf.indent():
                        err_gen(buf, f'Oneof {oneof.name} is required')
        for field in message.DESCRIPTOR.fields:
            if not field.containing_oneof:
                field_gen(buf, field, ['p', field.name], [field.name])
        if len(buf._lines) == pos:
            buf.add('pass')
    return buf.content()


def _validate_disabled(message: Message):
    pass


_validators = {}


def validate(message: Message):
    key = message.DESCRIPTOR.full_name
    func = _validators.get(key, None)
    if func is None:
        source = file_gen(message)
        if source is not None:
            locals_ = {}
            exec(source, CTX, locals_)
            func = _validators[key] = locals_['_validate']
        else:
            func = _validators[key] = _validate_disabled
    return func(message)


CTX = {
    're': re,
    ValidationError.__name__: ValidationError,
    'fmt': Format(),
    'sec': sec,
    'now': now,
    'dec': dec,
    'Counter': Counter,
    validate.__name__: validate,
}



import asyncio

from .. import net_pb2

from .base import Wire, WaitMixin


class MonitorWire(WaitMixin, Wire):
    """

    .. wire:: harness.wires.aiomonitor.MonitorWire
      :type: output
      :runtime: python
      :config: harness.net.Server
      :requirements: aiomonitor

    """
    _monitor = None

    def configure(self, value: net_pb2.Server):
        assert isinstance(value, net_pb2.Server), type(value)

        from aiomonitor import Monitor, MONITOR_HOST, MONITOR_PORT

        self._monitor = Monitor(
            loop=asyncio.get_event_loop(),
            host=value.bind.host or MONITOR_HOST,
            port=value.bind.port or MONITOR_PORT,
        )

    async def __aenter__(self):
        self._monitor.start()

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        self._monitor.close()



from typing import List
from dataclasses import dataclass

from google.protobuf.descriptor_pb2 import DescriptorProto, FieldDescriptorProto

from validate.validate_pb2 import FieldRules

from .runtime import validate
from .wire_pb2 import Wire, Service


@dataclass
class WireSpec:
    name: str
    type: str
    value: Wire
    optional: bool


@dataclass
class ConfigSpec:
    service: Service
    wires: List[WireSpec]


def translate_descriptor_proto(config_descriptor_proto) -> ConfigSpec:
    for _, option in config_descriptor_proto.options.ListFields():
        if isinstance(option, Service):
            service = option
            break
    else:
        service = Service()
    validate(service)

    wires = []
    for field in config_descriptor_proto.field:
        wire_option = next(
            (option for _, option in field.options.ListFields()
             if isinstance(option, Wire)),
            None,
        )
        if wire_option:
            assert field.type == FieldDescriptorProto.TYPE_MESSAGE, field.type
            validate(wire_option)
            type_ = field.type_name[1:]
            rules_option = next(
                (option for _, option in field.options.ListFields()
                 if isinstance(option, FieldRules)),
                None,
            )
            optional = not (rules_option and rules_option.message.required)
            wires.append(WireSpec(field.name, type_, wire_option, optional))
    return ConfigSpec(service=service, wires=wires)


def translate_descriptor(config_descriptor) -> ConfigSpec:
    proto = DescriptorProto()
    config_descriptor.CopyToProto(proto)
    return translate_descriptor_proto(proto)



import pathlib
import pkg_resources


def proto_path(args):
    wire_proto_path = pkg_resources.resource_filename('harness', 'wire.proto')
    print(pathlib.Path(wire_proto_path).parent.parent)


def add_commands(subparsers):
    check_parser = subparsers.add_parser('proto-path')
    check_parser.set_defaults(func=proto_path)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: harness/logging.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='harness/logging.proto',
  package='harness.logging',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x15harness/logging.proto\x12\x0fharness.logging\"\x83\x01\n\x07\x43onsole\x12/\n\x06stream\x18\x01 \x01(\x0e\x32\x1f.harness.logging.Console.Stream\x12%\n\x05level\x18\x02 \x01(\x0e\x32\x16.harness.logging.Level\" \n\x06Stream\x12\n\n\x06STDERR\x10\x00\x12\n\n\x06STDOUT\x10\x01\"\xf3\x01\n\x06Syslog\x12\x0b\n\x03\x61pp\x18\x01 \x01(\t\x12\x32\n\x08\x66\x61\x63ility\x18\x02 \x01(\x0e\x32 .harness.logging.Syslog.Facility\x12%\n\x05level\x18\x03 \x01(\x0e\x32\x16.harness.logging.Level\"\x80\x01\n\x08\x46\x61\x63ility\x12\n\n\x06NOTSET\x10\x00\x12\x08\n\x04USER\x10\x02\x12\n\n\x06LOCAL0\x10\x11\x12\n\n\x06LOCAL1\x10\x12\x12\n\n\x06LOCAL2\x10\x13\x12\n\n\x06LOCAL3\x10\x14\x12\n\n\x06LOCAL4\x10\x15\x12\n\n\x06LOCAL5\x10\x16\x12\n\n\x06LOCAL6\x10\x17\x12\n\n\x06LOCAL7\x10\x18*N\n\x05Level\x12\n\n\x06NOTSET\x10\x00\x12\t\n\x05\x44\x45\x42UG\x10\x01\x12\x08\n\x04INFO\x10\x02\x12\x0b\n\x07WARNING\x10\x03\x12\t\n\x05\x45RROR\x10\x04\x12\x0c\n\x08\x43RITICAL\x10\x05\x62\x06proto3'
)

_LEVEL = _descriptor.EnumDescriptor(
  name='Level',
  full_name='harness.logging.Level',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NOTSET', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DEBUG', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INFO', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WARNING', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CRITICAL', index=5, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=422,
  serialized_end=500,
)
_sym_db.RegisterEnumDescriptor(_LEVEL)

Level = enum_type_wrapper.EnumTypeWrapper(_LEVEL)
NOTSET = 0
DEBUG = 1
INFO = 2
WARNING = 3
ERROR = 4
CRITICAL = 5


_CONSOLE_STREAM = _descriptor.EnumDescriptor(
  name='Stream',
  full_name='harness.logging.Console.Stream',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='STDERR', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STDOUT', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=142,
  serialized_end=174,
)
_sym_db.RegisterEnumDescriptor(_CONSOLE_STREAM)

_SYSLOG_FACILITY = _descriptor.EnumDescriptor(
  name='Facility',
  full_name='harness.logging.Syslog.Facility',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NOTSET', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='USER', index=1, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOCAL0', index=2, number=17,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOCAL1', index=3, number=18,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOCAL2', index=4, number=19,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOCAL3', index=5, number=20,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOCAL4', index=6, number=21,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOCAL5', index=7, number=22,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOCAL6', index=8, number=23,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOCAL7', index=9, number=24,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=292,
  serialized_end=420,
)
_sym_db.RegisterEnumDescriptor(_SYSLOG_FACILITY)


_CONSOLE = _descriptor.Descriptor(
  name='Console',
  full_name='harness.logging.Console',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stream', full_name='harness.logging.Console.stream', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='level', full_name='harness.logging.Console.level', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _CONSOLE_STREAM,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=43,
  serialized_end=174,
)


_SYSLOG = _descriptor.Descriptor(
  name='Syslog',
  full_name='harness.logging.Syslog',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='app', full_name='harness.logging.Syslog.app', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='facility', full_name='harness.logging.Syslog.facility', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='level', full_name='harness.logging.Syslog.level', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _SYSLOG_FACILITY,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=177,
  serialized_end=420,
)

_CONSOLE.fields_by_name['stream'].enum_type = _CONSOLE_STREAM
_CONSOLE.fields_by_name['level'].enum_type = _LEVEL
_CONSOLE_STREAM.containing_type = _CONSOLE
_SYSLOG.fields_by_name['facility'].enum_type = _SYSLOG_FACILITY
_SYSLOG.fields_by_name['level'].enum_type = _LEVEL
_SYSLOG_FACILITY.containing_type = _SYSLOG
DESCRIPTOR.message_types_by_name['Console'] = _CONSOLE
DESCRIPTOR.message_types_by_name['Syslog'] = _SYSLOG
DESCRIPTOR.enum_types_by_name['Level'] = _LEVEL
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Console = _reflection.GeneratedProtocolMessageType('Console', (_message.Message,), {
  'DESCRIPTOR' : _CONSOLE,
  '__module__' : 'harness.logging_pb2'
  # @@protoc_insertion_point(class_scope:harness.logging.Console)
  })
_sym_db.RegisterMessage(Console)

Syslog = _reflection.GeneratedProtocolMessageType('Syslog', (_message.Message,), {
  'DESCRIPTOR' : _SYSLOG,
  '__module__' : 'harness.logging_pb2'
  # @@protoc_insertion_point(class_scope:harness.logging.Syslog)
  })
_sym_db.RegisterMessage(Syslog)


# @@protoc_insertion_point(module_scope)



import asyncio


class Wire:

    def configure(self, value):
        pass

    async def __aenter__(self):
        pass

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        self.close()
        await self.wait_closed()

    def close(self):
        pass

    async def wait_closed(self):
        pass


class WaitMixin:
    _event: asyncio.Event

    def close(self):
        if not hasattr(self, '_event'):
            self._event = asyncio.Event()
        self._event.set()

    async def wait_closed(self):
        if not hasattr(self, '_event'):
            self._event = asyncio.Event()
        await self._event.wait()



import os
import sys

from google.protobuf.compiler.plugin_pb2 import CodeGeneratorRequest
from google.protobuf.compiler.plugin_pb2 import CodeGeneratorResponse

from ..config import translate_descriptor_proto

from . import python


RUNTIMES = {
    'python': python.render,
}


class ConfigurationError(ValueError):
    pass


def process_file(renderer, proto_file, response):
    for config_message in proto_file.message_type:
        if config_message.name == 'Configuration':
            break
    else:
        raise ConfigurationError('Missing configuration message')

    config_spec = translate_descriptor_proto(config_message)
    for name, content in renderer(proto_file.name, config_spec):
        f = response.file.add()
        f.name = name
        f.content = content


def process(request: CodeGeneratorRequest) -> CodeGeneratorResponse:
    params = {}
    for pair in request.parameter.split(','):
        key, _, value = pair.partition('=')
        params[key] = value

    if 'runtime' not in params:
        raise ConfigurationError('Runtime parameter is not specified')
    runtime = params['runtime']
    if runtime not in RUNTIMES:
        raise ConfigurationError(f'Unknown runtime: {runtime}')
    renderer = RUNTIMES[runtime]

    files_to_generate = set(request.file_to_generate)

    response = CodeGeneratorResponse()
    for pf in request.proto_file:
        if pf.name in files_to_generate:
            process_file(renderer, pf, response)
    return response


def main() -> None:
    with os.fdopen(sys.stdin.fileno(), 'rb') as inp:
        request = CodeGeneratorRequest.FromString(inp.read())
    response = process(request)
    with os.fdopen(sys.stdout.fileno(), 'wb') as out:
        out.write(response.SerializeToString())



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: harness/net.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from harness import wire_pb2 as harness_dot_wire__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='harness/net.proto',
  package='harness.net',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x11harness/net.proto\x12\x0bharness.net\x1a\x12harness/wire.proto\x1a\x17validate/validate.proto\"6\n\x06Socket\x12\x15\n\x04host\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x15\n\x04port\x18\x02 \x01(\rB\x07\xfa\x42\x04*\x02 \x00\"\"\n\x04Pipe\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04mode\x18\x02 \x01(\r\"H\n\x07\x41\x64\x64ress\x12%\n\x06socket\x18\x01 \x01(\x0b\x32\x13.harness.net.SocketH\x00\x12\x0e\n\x04pipe\x18\x02 \x01(\tH\x00\x42\x06\n\x04type\"2\n\x06Server\x12(\n\x04\x62ind\x18\x01 \x01(\x0b\x32\x13.harness.net.SocketB\x05\x92}\x02\x08\x00\x62\x06proto3'
  ,
  dependencies=[harness_dot_wire__pb2.DESCRIPTOR,validate_dot_validate__pb2.DESCRIPTOR,])




_SOCKET = _descriptor.Descriptor(
  name='Socket',
  full_name='harness.net.Socket',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='host', full_name='harness.net.Socket.host', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\004r\002\020\001', file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='port', full_name='harness.net.Socket.port', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\004*\002 \000', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=79,
  serialized_end=133,
)


_PIPE = _descriptor.Descriptor(
  name='Pipe',
  full_name='harness.net.Pipe',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='harness.net.Pipe.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mode', full_name='harness.net.Pipe.mode', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=135,
  serialized_end=169,
)


_ADDRESS = _descriptor.Descriptor(
  name='Address',
  full_name='harness.net.Address',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='socket', full_name='harness.net.Address.socket', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pipe', full_name='harness.net.Address.pipe', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='type', full_name='harness.net.Address.type',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=171,
  serialized_end=243,
)


_SERVER = _descriptor.Descriptor(
  name='Server',
  full_name='harness.net.Server',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='bind', full_name='harness.net.Server.bind', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222}\002\010\000', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=245,
  serialized_end=295,
)

_ADDRESS.fields_by_name['socket'].message_type = _SOCKET
_ADDRESS.oneofs_by_name['type'].fields.append(
  _ADDRESS.fields_by_name['socket'])
_ADDRESS.fields_by_name['socket'].containing_oneof = _ADDRESS.oneofs_by_name['type']
_ADDRESS.oneofs_by_name['type'].fields.append(
  _ADDRESS.fields_by_name['pipe'])
_ADDRESS.fields_by_name['pipe'].containing_oneof = _ADDRESS.oneofs_by_name['type']
_SERVER.fields_by_name['bind'].message_type = _SOCKET
DESCRIPTOR.message_types_by_name['Socket'] = _SOCKET
DESCRIPTOR.message_types_by_name['Pipe'] = _PIPE
DESCRIPTOR.message_types_by_name['Address'] = _ADDRESS
DESCRIPTOR.message_types_by_name['Server'] = _SERVER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Socket = _reflection.GeneratedProtocolMessageType('Socket', (_message.Message,), {
  'DESCRIPTOR' : _SOCKET,
  '__module__' : 'harness.net_pb2'
  # @@protoc_insertion_point(class_scope:harness.net.Socket)
  })
_sym_db.RegisterMessage(Socket)

Pipe = _reflection.GeneratedProtocolMessageType('Pipe', (_message.Message,), {
  'DESCRIPTOR' : _PIPE,
  '__module__' : 'harness.net_pb2'
  # @@protoc_insertion_point(class_scope:harness.net.Pipe)
  })
_sym_db.RegisterMessage(Pipe)

Address = _reflection.GeneratedProtocolMessageType('Address', (_message.Message,), {
  'DESCRIPTOR' : _ADDRESS,
  '__module__' : 'harness.net_pb2'
  # @@protoc_insertion_point(class_scope:harness.net.Address)
  })
_sym_db.RegisterMessage(Address)

Server = _reflection.GeneratedProtocolMessageType('Server', (_message.Message,), {
  'DESCRIPTOR' : _SERVER,
  '__module__' : 'harness.net_pb2'
  # @@protoc_insertion_point(class_scope:harness.net.Server)
  })
_sym_db.RegisterMessage(Server)


_SOCKET.fields_by_name['host']._options = None
_SOCKET.fields_by_name['port']._options = None
_SERVER.fields_by_name['bind']._options = None
# @@protoc_insertion_point(module_scope)



import hashlib
from typing import List, Optional
from base64 import b64encode
from argparse import FileType
from itertools import chain
from contextlib import closing
from dataclasses import dataclass

import yaml

from google.protobuf.message import Message
from google.protobuf.descriptor import FieldDescriptor
from google.protobuf.json_format import ParseDict
from google.protobuf.message_factory import GetMessages

from .. import wire_pb2
from ..config import translate_descriptor, WireSpec
from ..net_pb2 import Socket
from ..runtime._utils import load_config
from ..runtime._validate import validate

from .utils import load_descriptor_set, get_configuration


@dataclass
class Runtime:
    entrypoint: List[str]


RUNTIMES = {
    'python': Runtime(
        entrypoint=['python3', '/app/entrypoint.py'],
    ),
}


def _ver(content_bytes):
    return hashlib.new('sha1', content_bytes).hexdigest()[:8]


@dataclass
class Socket:
    host: str
    port: int
    _protocol: wire_pb2.Mark.Protocol

    def is_tcp(self):
        return self._protocol == wire_pb2.Mark.TCP

    def is_http(self):
        return self._protocol == wire_pb2.Mark.HTTP

    def is_grpc(self):
        return self._protocol == wire_pb2.Mark.GRPC


@dataclass
class Input:
    _wire: WireSpec
    socket: Optional[Socket] = None

    @property
    def name(self):
        return self._wire.name

    def is_localhost(self):
        return self._wire.value.input.reach == wire_pb2.Input.LOCALHOST

    def is_namespace(self):
        return self._wire.value.input.reach == wire_pb2.Input.NAMESPACE

    def is_cluster(self):
        return self._wire.value.input.reach == wire_pb2.Input.CLUSTER

    def is_external(self):
        return self._wire.value.input.reach == wire_pb2.Input.EXTERNAL


@dataclass
class Output:
    _wire: WireSpec
    socket: Optional[Socket] = None

    @property
    def name(self):
        return self._wire.name

    def is_private(self):
        return self._wire.value.output.expose == wire_pb2.Output.PRIVATE

    def is_internal(self):
        return self._wire.value.output.expose == wire_pb2.Output.INTERNAL

    def is_public(self):
        return self._wire.value.output.expose == wire_pb2.Output.PUBLIC

    def is_headless(self):
        return self._wire.value.output.expose == wire_pb2.Output.HEADLESS


@dataclass
class HostPath:
    name: str
    path: str
    type: str


@dataclass
class Context:
    _container: wire_pb2.Service.Container

    name: str
    inputs: List[Input]
    outputs: List[Output]
    host_paths: List[HostPath]
    entrypoint: List[str]
    config_content: str
    config_hash: str
    secret_merge_content: Optional[str]
    secret_merge_hash: Optional[str]
    secret_patch_content: Optional[str]
    secret_patch_hash: Optional[str]

    # pass-through
    version: str
    instance: Optional[str]
    namespace: str
    base_domain: Optional[str]

    # constants
    config_volume = 'config'
    secret_merge_volume = 'config-merge'
    secret_patch_volume = 'config-patch'

    @property
    def repository(self):
        return self._container.repository

    def resources(self):
        data = dict()
        if self._container.resources.requests:
            requests = data['requests'] = {}
            if self._container.resources.requests.cpu:
                requests['cpu'] = self._container.resources.requests.cpu
            if self._container.resources.requests.memory:
                requests['memory'] = self._container.resources.requests.memory
        if self._container.resources.limits:
            limits = data['limits'] = {}
            if self._container.resources.limits.cpu:
                limits['cpu'] = self._container.resources.limits.cpu
            if self._container.resources.limits.memory:
                limits['memory'] = self._container.resources.limits.memory
        return data

    @property
    def config_name(self):
        return self.full_name('config', self.config_hash)

    @property
    def secret_merge_name(self):
        return self.full_name('config-merge', self.secret_merge_hash)

    @property
    def secret_patch_name(self):
        return self.full_name('config-patch', self.secret_merge_hash)

    def full_name(self, *suffix: str):
        parts = [self.name]
        if self.instance is not None:
            parts.append(self.instance)
        parts.extend(suffix)
        return '-'.join(parts)

    def labels(self):
        labels = {'app.kubernetes.io/name': self.name}
        if self.instance is not None:
            labels['app.kubernetes.io/instance'] = self.instance
        return labels

    @property
    def public_domain(self):
        if self.base_domain is not None:
            if self.instance is None:
                sub_domain = self.name
            else:
                sub_domain = f'{self.name}-{self.instance}'
            return f'{sub_domain}.{self.base_domain}'


def get_socket(
    wire: WireSpec, message_classes, config: Message,
) -> Optional[Socket]:
    if wire.optional:
        if not config.HasField(wire.name):
            return None
    else:
        assert config.HasField(wire.name), f'Config unset: {wire.name}'

    wire_type = message_classes[wire.type]
    socket_type_fd = next(
        (fd for fd in wire_type.DESCRIPTOR.fields
         if fd.type == FieldDescriptor.TYPE_MESSAGE
         and fd.message_type.full_name == 'harness.net.Socket'),
        None,
    )
    if socket_type_fd is None:
        return None

    protocol = wire_pb2.Mark.TCP
    for _, option in socket_type_fd.GetOptions().ListFields():
        if isinstance(option, wire_pb2.Mark):
            protocol = option.protocol

    wire_value = getattr(config, wire.name)
    socket_value = getattr(wire_value, socket_type_fd.name)
    return Socket(
        host=socket_value.host,
        port=socket_value.port,
        _protocol=protocol,
    )


def get_context(
    *,
    proto_file,
    proto_path,
    runtime,
    config_bytes,
    secret_merge_bytes=None,
    secret_patch_bytes=None,
    passthrough=None,
):
    runtime = RUNTIMES[runtime]

    descriptor_set = load_descriptor_set(proto_file, proto_path)
    file_descriptor, = (f for f in descriptor_set.file
                        if proto_file.endswith(f.name))

    config_hash = _ver(config_bytes)
    config_content = config_bytes.decode('utf-8')

    if secret_merge_bytes is not None:
        secret_merge_hash = _ver(secret_merge_bytes)
        secret_merge_content = secret_merge_bytes.decode('utf-8')
    else:
        secret_merge_hash = None
        secret_merge_content = None

    if secret_patch_bytes is not None:
        secret_patch_hash = _ver(secret_patch_bytes)
        secret_patch_content = secret_patch_bytes.decode('utf-8')
    else:
        secret_patch_hash = None
        secret_patch_content = None

    message_classes = GetMessages(descriptor_set.file)

    config_descriptor = get_configuration(file_descriptor)
    if config_descriptor is None:
        raise Exception(f'Configuration message not found in the {proto_file}')

    config_data = load_config(
        config_content, secret_merge_content, secret_patch_content,
    )

    config_full_name = '.'.join(filter(None, (
        file_descriptor.package,
        config_descriptor.name,
    )))
    config_cls = message_classes[config_full_name]
    config = config_cls()
    ParseDict(config_data, config)
    validate(config)

    config_spec = translate_descriptor(config_cls.DESCRIPTOR)

    inputs = []
    outputs = []
    for wire in config_spec.wires:
        wire_type = wire.value.WhichOneof('type')
        if wire_type == 'input':
            inputs.append(Input(
                _wire=wire,
                socket=get_socket(wire, message_classes, config),
            ))
        elif wire_type == 'output':
            outputs.append(Output(
                _wire=wire,
                socket=get_socket(wire, message_classes, config),
            ))

    host_paths = []
    for wire in config_spec.wires:
        if wire.type == 'harness.logging.Syslog':
            host_paths.append(HostPath(
                name='syslog',
                path='/dev/log',
                type='Socket',
            ))

    assert config_spec.service.name
    return Context(
        name=config_spec.service.name,
        inputs=inputs,
        outputs=outputs,
        _container=config_spec.service.container,
        host_paths=host_paths,
        entrypoint=runtime.entrypoint,
        config_content=config_content,
        config_hash=config_hash,
        secret_merge_content=secret_merge_content,
        secret_merge_hash=secret_merge_hash,
        secret_patch_content=secret_patch_content,
        secret_patch_hash=secret_patch_hash,
        **(passthrough or {})
    )


def istio_type(socket: Socket):
    if socket.is_tcp():
        return 'TCP'
    elif socket.is_http():
        return 'HTTP'
    elif socket.is_grpc():
        return 'GRPC'
    else:
        return 'TCP'


def istio_name(socket, suffix):
    if socket.is_tcp():
        name = 'tcp'
    elif socket.is_http():
        name = 'http'
    elif socket.is_grpc():
        name = 'grpc'
    else:
        name = 'tcp'
    return name + '-' + suffix


def gen_deployments(ctx: 'Context'):
    labels = ctx.labels()
    labels['app.kubernetes.io/version'] = ctx.version

    command = ctx.entrypoint + ['/etc/config/config.yaml']
    if ctx.secret_merge_content is not None:
        command.extend(['--merge', '/etc/config-merge/config.yaml'])
    if ctx.secret_patch_content is not None:
        command.extend(['--patch', '/etc/config-patch/config.yaml'])

    container = dict(
        name='app',
        image=f'{ctx.repository}:{ctx.version}',
        command=command,
        securityContext=dict(
            runAsNonRoot=True,
        ),
        volumeMounts=[
            dict(
                mountPath='/etc/config',
                name=ctx.config_volume,
            ),
        ],
    )
    if ctx.secret_merge_content is not None:
        container['volumeMounts'].append(dict(
            mountPath='/etc/config-merge',
            name=ctx.secret_merge_volume,
        ))
    if ctx.secret_patch_content is not None:
        container['volumeMounts'].append(dict(
            mountPath='/etc/config-patch',
            name=ctx.secret_patch_volume,
        ))
    for host_path in ctx.host_paths:
        container['volumeMounts'].append(dict(
            mountPath=host_path.path,
            name=host_path.name,
        ))

    if ctx.outputs:
        container_ports = container['ports'] = []
        for wire in ctx.outputs:
            if wire.socket is not None:
                container_port = dict(
                    name=istio_name(wire.socket, wire.name),
                    containerPort=wire.socket.port,
                )
                container_ports.append(container_port)

    for wire in ctx.outputs:
        if not wire.is_public() and not wire.is_internal() or wire.socket is None:
            continue
        if wire.socket.is_http():
            container['readinessProbe'] = dict(
                httpGet=dict(
                    path='/health/ready',
                    port=istio_name(wire.socket, wire.name),
                ),
            )
            container['livenessProbe'] = dict(
                httpGet=dict(
                    path='/health/live',
                    port=istio_name(wire.socket, wire.name),
                ),
            )
        elif wire.socket.is_grpc():
            container['readinessProbe'] = dict(
                exec=dict(
                    command=[
                        'grpc_health_probe',
                        '-addr',
                        f'localhost:{wire.socket.port}',
                    ],
                ),
            )

    resources = ctx.resources()
    if resources:
        container['resources'] = resources

    volumes = [
        dict(
            name=ctx.config_volume,
            configMap=dict(
                name=ctx.config_name,
            ),
        ),
    ]
    if ctx.secret_merge_content is not None:
        volumes.append(dict(
            name=ctx.secret_merge_volume,
            secret=dict(
                name=ctx.secret_merge_name,
            ),
        ))
    if ctx.secret_patch_content is not None:
        volumes.append(dict(
            name=ctx.secret_patch_volume,
            secret=dict(
                name=ctx.secret_patch_name,
            ),
        ))
    for host_path in ctx.host_paths:
        volumes.append(dict(
            name=host_path.name,
            hostPath=dict(
                path=host_path.path,
                type=host_path.type,
            ),
        ))

    yield dict(
        apiVersion='apps/v1',
        kind='Deployment',
        metadata=dict(
            name=ctx.full_name(),
            namespace=ctx.namespace,
        ),
        spec=dict(
            selector=dict(
                matchLabels=labels,
            ),
            template=dict(
                metadata=dict(
                    labels=labels,
                ),
                spec=dict(
                    containers=[container],
                    volumes=volumes,
                ),
            ),
        ),
    )


def gen_services(ctx: 'Context'):
    def port(value: Output):
        return dict(
            name=istio_name(value.socket, value.name),
            port=value.socket.port,
            targetPort=istio_name(value.socket, value.name),
        )

    regular_outputs = [
        i for i in ctx.outputs
        if (i.is_internal() or i.is_public()) and i.socket is not None
    ]
    headless_outputs = [
        i for i in ctx.outputs
        if i.is_headless() and i.socket is not None
    ]
    if regular_outputs:
        yield dict(
            apiVersion='v1',
            kind='Service',
            metadata=dict(
                name=ctx.full_name(),
                namespace=ctx.namespace,
            ),
            spec=dict(
                ports=[port(i) for i in regular_outputs],
                selector=ctx.labels(),
            ),
        )
    if headless_outputs:
        yield dict(
            apiVersion='v1',
            kind='Service',
            metadata=dict(
                name=ctx.full_name('headless'),
                namespace=ctx.namespace,
            ),
            spec=dict(
                clusterIP='None',
                ports=[port(i) for i in headless_outputs],
                selector=ctx.labels(),
            ),
        )


def gen_virtualservices(ctx: 'Context'):
    internal_outputs = [
        i for i in ctx.outputs if i.is_internal() and i.socket is not None
    ]
    public_outputs = [
        i for i in ctx.outputs if i.is_public() and i.socket is not None
    ]
    if not internal_outputs and not public_outputs:
        return

    hosts = []
    if internal_outputs:
        hosts.append(ctx.full_name())
    if public_outputs and ctx.public_domain:
        hosts.append(ctx.public_domain)

    spec = dict(
        hosts=hosts,
        http=[
            dict(
                match=[dict(port=i.socket.port)],
                route=[
                    dict(
                        destination=dict(
                            host=ctx.full_name(),
                            port=dict(number=i.socket.port),
                        )
                    )
                ],
            )
            for i in chain(public_outputs, internal_outputs)
        ],
    )
    if public_outputs:
        spec['gateways'] = [ctx.full_name(), 'mesh']

    yield dict(
        apiVersion='networking.istio.io/v1alpha3',
        kind='VirtualService',
        metadata=dict(
            name=ctx.full_name(),
            namespace=ctx.namespace,
        ),
        spec=spec,
    )


def gen_gateways(ctx: 'Context'):
    public_outputs = [out for out in ctx.outputs if out.is_public()]
    if not public_outputs or not ctx.public_domain:
        return

    yield dict(
        apiVersion='networking.istio.io/v1alpha3',
        kind='Gateway',
        metadata=dict(
            name=ctx.full_name(),
            namespace=ctx.namespace,
        ),
        spec=dict(
            selector={'istio': 'ingressgateway'},
            servers=[
                dict(
                    port=dict(
                        number=out.socket.port,
                        protocol=istio_type(out.socket),
                    ),
                    hosts=[ctx.public_domain],
                )
                for out in public_outputs
            ],
        ),
    )


def gen_sidecars(ctx: 'Context'):
    hosts = ['istio-system/*']
    for inp in ctx.inputs:
        if inp.socket is None:
            continue
        if inp.is_namespace():
            assert ctx.namespace
            host = inp.socket.host
            hosts.append(f'./{host}.{ctx.namespace}.svc.cluster.local')
        elif inp.is_cluster():
            host = inp.socket.host
            hosts.append(f'*/{host}')
        elif inp.is_external():
            host = inp.socket.host
            hosts.append(f'./{host}')
        else:
            continue
    yield dict(
        apiVersion='networking.istio.io/v1alpha3',
        kind='Sidecar',
        metadata=dict(
            name=ctx.full_name(),
            namespace=ctx.namespace,
        ),
        spec=dict(
            egress=[dict(
                hosts=hosts,
            )],
            workloadSelector=dict(
                labels=ctx.labels(),
            ),
        ),
    )


def gen_serviceentries(ctx: 'Context'):
    for inp in ctx.inputs:
        if not inp.is_external():
            continue
        yield dict(
            apiVersion='networking.istio.io/v1alpha3',
            kind='ServiceEntry',
            metadata=dict(
                name=ctx.full_name(inp.name),
                namespace=ctx.namespace,
            ),
            spec=dict(
                hosts=[inp.socket.host],
                location='MESH_EXTERNAL',
                ports=[dict(
                    number=inp.socket.port,
                    protocol=istio_type(inp.socket),
                )],
            ),
        )


def gen_configmaps(ctx: 'Context'):
    yield dict(
        apiVersion='v1',
        kind='ConfigMap',
        metadata=dict(
            name=ctx.config_name,
            namespace=ctx.namespace,
        ),
        data={
            'config.yaml': ctx.config_content,
        },
    )


def gen_secrets(ctx: 'Context'):
    def b64(string):
        return b64encode(string.encode('utf-8')).decode('ascii')
    if ctx.secret_merge_content is not None:
        yield dict(
            apiVersion='v1',
            kind='Secret',
            type='Opaque',
            metadata=dict(
                name=ctx.secret_merge_name,
                namespace=ctx.namespace,
            ),
            data={
                'config.yaml': b64(ctx.secret_merge_content),
            },
        )
    if ctx.secret_patch_content is not None:
        yield dict(
            apiVersion='v1',
            kind='Secret',
            type='Opaque',
            metadata=dict(
                name=ctx.secret_patch_name,
                namespace=ctx.namespace,
            ),
            data={
                'config.yaml': b64(ctx.secret_patch_content),
            },
        )


def kube_gen(args):
    # configuration
    with closing(args.config):
        config_bytes = args.config.read()

    # merge
    if args.secret_merge:
        with closing(args.secret_merge):
            secret_merge_bytes = args.secret_merge.read()
    else:
        secret_merge_bytes = None

    # patch
    if args.secret_patch:
        with closing(args.secret_patch):
            secret_patch_bytes = args.secret_patch.read()
    else:
        secret_patch_bytes = None

    ctx = get_context(
        proto_file=args.proto,
        proto_path=args.proto_path,
        runtime=args.runtime,
        config_bytes=config_bytes,
        secret_merge_bytes=secret_merge_bytes,
        secret_patch_bytes=secret_patch_bytes,
        passthrough=dict(
            version=args.version,
            instance=args.instance,
            namespace=args.namespace,
            base_domain=args.base_domain,
        ),
    )
    resources = list(chain(
        gen_configmaps(ctx),
        gen_secrets(ctx),
        gen_deployments(ctx),
        gen_services(ctx),
        gen_gateways(ctx),
        gen_virtualservices(ctx),
        gen_serviceentries(ctx),
        gen_sidecars(ctx),
    ))
    print(yaml.dump_all(resources))


def add_commands(subparsers):
    kube_gen_parser = subparsers.add_parser('kube-gen')
    kube_gen_parser.add_argument('-I', '--proto-path', action='append')
    kube_gen_parser.add_argument('runtime', choices=frozenset(RUNTIMES.keys()))
    kube_gen_parser.add_argument('proto')
    kube_gen_parser.add_argument('config', type=FileType('rb'))
    kube_gen_parser.add_argument('version')
    kube_gen_parser.add_argument('--instance', default=None)
    kube_gen_parser.add_argument('--namespace', default='default')
    kube_gen_parser.add_argument('--secret-merge', type=FileType('rb'),
                                 default=None)
    kube_gen_parser.add_argument('--secret-patch', type=FileType('rb'),
                                 default=None)
    kube_gen_parser.add_argument('--base-domain', default=None)
    kube_gen_parser.set_defaults(func=kube_gen)



import logging
from http import HTTPStatus
from typing import Callable, Awaitable, List, Dict, Optional
from logging import Logger

from aiohttp.web import Application, AppRunner, TCPSite, Request, Response
from aiohttp.web import HTTPException, middleware

from opentelemetry.trace import get_tracer, SpanKind
from opentelemetry.propagators import extract
from opentelemetry.trace.status import Status, StatusCanonicalCode

from ... import http_pb2

from ..base import Wire, WaitMixin


_log = logging.getLogger(__name__)


def _headers_getter(request: Request, header_name: str) -> List[str]:
    return request.headers.getall(header_name, [])


_HTTP_STATUS_TO_CODE_MAP: Dict[int, StatusCanonicalCode] = {
    # 401
    HTTPStatus.UNAUTHORIZED.value: StatusCanonicalCode.UNAUTHENTICATED,
    # 403
    HTTPStatus.FORBIDDEN.value: StatusCanonicalCode.PERMISSION_DENIED,
    # 404
    HTTPStatus.NOT_FOUND.value: StatusCanonicalCode.NOT_FOUND,
    # 429
    HTTPStatus.TOO_MANY_REQUESTS.value: StatusCanonicalCode.RESOURCE_EXHAUSTED,
    # 501
    HTTPStatus.NOT_IMPLEMENTED.value: StatusCanonicalCode.UNIMPLEMENTED,
    # 503
    HTTPStatus.SERVICE_UNAVAILABLE.value: StatusCanonicalCode.UNAVAILABLE,
    # 504
    HTTPStatus.GATEWAY_TIMEOUT.value: StatusCanonicalCode.DEADLINE_EXCEEDED,
}


def _status_to_canonical_code(status_code: int):
    try:
        return _HTTP_STATUS_TO_CODE_MAP[status_code]
    except KeyError:
        if status_code < 100:
            return StatusCanonicalCode.UNKNOWN
        elif status_code < 400:
            return StatusCanonicalCode.OK
        elif status_code < 500:
            return StatusCanonicalCode.INVALID_ARGUMENT
        elif status_code < 600:
            return StatusCanonicalCode.INTERNAL
        else:
            return StatusCanonicalCode.UNKNOWN


@middleware
async def _opentracing_middleware(
    request: Request,
    handler: Callable[[Request], Awaitable[Response]],
) -> Response:
    tracer = get_tracer(__name__)
    parent_span = extract(_headers_getter, request)
    with tracer.start_as_current_span(
        request.path,
        parent_span,
        kind=SpanKind.SERVER,
        attributes={
            "component": "http",
            "http.method": request.method,
            "http.scheme": request.scheme,
            "http.host": request.host,
        },
    ) as span:
        try:
            response = await handler(request)
        except HTTPException as e:
            code = _status_to_canonical_code(e.status)
            span.set_status(Status(code))
            span.set_attribute("http.status_text", e.reason)
            raise
        else:
            code = _status_to_canonical_code(response.status)
            span.set_status(Status(code))
            span.set_attribute("http.status_text", response.reason)
            return response


class ServerWire(WaitMixin, Wire):
    """
    Output wire to start HTTP server and serve aiohttp application.

    .. wire:: harness.wires.aiohttp.web.ServerWire
      :type: output
      :runtime: python
      :config: harness.http.Server
      :requirements: aiohttp[speedups]

    """
    _runner: AppRunner
    _site: TCPSite
    _site_factory: Callable[[], TCPSite]

    def __init__(
        self, app: 'Application', *, access_log: Optional[Logger] = None,
    ) -> None:
        """
        :param app: configured :py:class:`aiohttp:aiohttp.web.Application`
            to run
        :param access_log: enable access logs by providing a logger instance
        """
        self._app = app
        self._access_log = access_log

    def configure(self, value: http_pb2.Server):
        assert isinstance(value, http_pb2.Server), type(value)

        self._app.middlewares.append(_opentracing_middleware)
        self._runner = AppRunner(self._app, access_log=self._access_log)
        self._site_factory = lambda: TCPSite(
            self._runner,
            value.bind.host,
            value.bind.port,
        )

    async def __aenter__(self):
        await self._runner.setup()
        self._site = self._site_factory()
        await self._site.start()
        _log.info('%s started: %s', self.__class__.__name__, self._site.name)

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self._runner.cleanup()



import os.path

from setuptools import setup, find_packages

with open(
    os.path.join(os.path.dirname(__file__), 'README.rst')
) as f:
    DESCRIPTION = f.read()

setup(
    name='harness',
    version='0.1.0rc3',
    description='Language-neutral microservices boilerplate as a code',
    long_description=DESCRIPTION,
    long_description_content_type='text/x-rst',
    author='Vladimir Magamedov',
    author_email='vladimir@magamedov.com',
    url='https://github.com/vmagamedov/harness',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    license='BSD-3-Clause',
    python_requires='>=3.7',
    install_requires=[
        'protobuf',
        'json-merge-patch',
        'jsonpatch',
        'opentelemetry-sdk==0.4a1',
    ],
    entry_points={
        'console_scripts': [
            'protoc-gen-harness=harness.plugin.main:main',
            'harness=harness.cli.main:main',
        ],
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3 :: Only',
    ],
)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: harness/http.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from harness import wire_pb2 as harness_dot_wire__pb2
from harness import net_pb2 as harness_dot_net__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='harness/http.proto',
  package='harness.http',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x12harness/http.proto\x12\x0charness.http\x1a\x12harness/wire.proto\x1a\x11harness/net.proto\"9\n\nConnection\x12+\n\x07\x61\x64\x64ress\x18\x01 \x01(\x0b\x32\x13.harness.net.SocketB\x05\x92}\x02\x08\x01\"2\n\x06Server\x12(\n\x04\x62ind\x18\x01 \x01(\x0b\x32\x13.harness.net.SocketB\x05\x92}\x02\x08\x01\x62\x06proto3'
  ,
  dependencies=[harness_dot_wire__pb2.DESCRIPTOR,harness_dot_net__pb2.DESCRIPTOR,])




_CONNECTION = _descriptor.Descriptor(
  name='Connection',
  full_name='harness.http.Connection',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='address', full_name='harness.http.Connection.address', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222}\002\010\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=75,
  serialized_end=132,
)


_SERVER = _descriptor.Descriptor(
  name='Server',
  full_name='harness.http.Server',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='bind', full_name='harness.http.Server.bind', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\222}\002\010\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=134,
  serialized_end=184,
)

_CONNECTION.fields_by_name['address'].message_type = harness_dot_net__pb2._SOCKET
_SERVER.fields_by_name['bind'].message_type = harness_dot_net__pb2._SOCKET
DESCRIPTOR.message_types_by_name['Connection'] = _CONNECTION
DESCRIPTOR.message_types_by_name['Server'] = _SERVER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Connection = _reflection.GeneratedProtocolMessageType('Connection', (_message.Message,), {
  'DESCRIPTOR' : _CONNECTION,
  '__module__' : 'harness.http_pb2'
  # @@protoc_insertion_point(class_scope:harness.http.Connection)
  })
_sym_db.RegisterMessage(Connection)

Server = _reflection.GeneratedProtocolMessageType('Server', (_message.Message,), {
  'DESCRIPTOR' : _SERVER,
  '__module__' : 'harness.http_pb2'
  # @@protoc_insertion_point(class_scope:harness.http.Server)
  })
_sym_db.RegisterMessage(Server)


_CONNECTION.fields_by_name['address']._options = None
_SERVER.fields_by_name['bind']._options = None
# @@protoc_insertion_point(module_scope)



import asyncio
import argparse
from typing import Generic, TypeVar, Callable, Awaitable, List, Type
from contextlib import AsyncExitStack
from dataclasses import fields

from grpclib.utils import graceful_exit
from google.protobuf.json_format import ParseDict

from ..wires.base import Wire

from ._utils import load_config
from ._validate import validate
from ._features import enable_metrics, enable_tracing


# Temporary decision to enable metrics and tracing by default. But maybe it
# would be better to make this [auto]configurable.
enable_metrics()
enable_tracing()


_CT = TypeVar('_CT')
_WI = TypeVar('_WI')
_WO = TypeVar('_WO')


class Runner(Generic[_CT, _WI, _WO]):

    def __init__(
        self,
        config_type: Type[_CT],
        wires_in_type: Type[_WI],
        wires_out_type: Type[_WO],
    ) -> None:
        self._config_type = config_type
        self._wires_in_type = wires_in_type
        self._wires_out_type = wires_out_type

        self._arg_parser = argparse.ArgumentParser()
        self._arg_parser.add_argument(
            'config',
            type=argparse.FileType('r', encoding='utf-8'),
            help='Configuration file in the YAML format',
        )
        self._arg_parser.add_argument(
            '--merge',
            default=None,
            type=argparse.FileType('r', encoding='utf-8'),
            help='Merge config with a file',
        )
        self._arg_parser.add_argument(
            '--patch',
            default=None,
            type=argparse.FileType('r', encoding='utf-8'),
            help='Patch config with a file',
        )

    async def _wrapper(
        self, main_func: Callable[[_CT, _WI], Awaitable[_WO]], config: _CT,
    ) -> None:
        async with AsyncExitStack() as stack:
            input_wires = {}
            for field in fields(self._wires_in_type):
                if config.HasField(field.name):
                    wire_config = getattr(config, field.name)
                    if isinstance(field.type, type) and issubclass(field.type, Wire):
                        wire_type = field.type
                    else:
                        # Optional[wire_type]
                        wire_type = field.type.__args__[0]
                        assert isinstance(wire_type, type) and issubclass(wire_type, Wire), type(wire_type)
                    wire = wire_type()
                    wire.configure(wire_config)
                    await stack.enter_async_context(wire)
                else:
                    if isinstance(field.type, type) and issubclass(field.type, Wire):
                        raise RuntimeError(f'Missing configuration for'
                                           f' required wire: {field.name}')
                    wire = None
                input_wires[field.name] = wire
            wires_in = self._wires_in_type(**input_wires)

            wires_out = await main_func(config, wires_in)

            if not isinstance(wires_out, self._wires_out_type):
                raise RuntimeError(
                    f'{main_func} returned invalid type: {type(wires_out)!r}; '
                    f'expected: {self._wires_out_type!r}'
                )

            waiters = set()
            output_wires = []
            for field in fields(self._wires_out_type):
                if not config.HasField(field.name):
                    continue
                wire = getattr(wires_out, field.name)
                wire_config = getattr(config, field.name)
                wire.configure(wire_config)
                await stack.enter_async_context(wire)
                waiters.add(wire.wait_closed())
                output_wires.append(wire)

            if output_wires:
                with graceful_exit(output_wires):
                    await asyncio.wait(
                        waiters, return_when=asyncio.FIRST_COMPLETED,
                    )

    def run(
        self, main_func: Callable[[_WI], Awaitable[_WO]], args: List[str],
    ) -> int:
        args = self._arg_parser.parse_args(args[1:])

        with args.config:
            config_content = args.config.read()

        if args.merge is not None:
            with args.merge:
                merge_content = args.merge.read()
        else:
            merge_content = None

        if args.patch is not None:
            with args.patch:
                patch_content = args.patch.read()
        else:
            patch_content = None

        config_data = load_config(config_content, merge_content, patch_content)
        config = self._config_type()
        ParseDict(config_data, config)
        validate(config)

        asyncio.run(self._wrapper(main_func, config))
        return 0



from apscheduler.jobstores.redis import RedisJobStore

from .... import redis_pb2

from ....wires.base import Wire


class RedisJobStoreWire(Wire):
    """

    .. wire:: harness.wires.apscheduler.jobstores.redis.RedisJobStoreWire
      :type: input
      :runtime: python
      :config: harness.redis.Connection
      :requirements: redis

    """
    job_store: 'RedisJobStore'

    def configure(self, value: redis_pb2.Connection):
        self.job_store = RedisJobStore(
            db=value.db, host=value.address.host, port=value.address.port,
        )

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        self.job_store.shutdown()



from asyncpg import Connection, create_pool
from asyncpg.pool import Pool
from opentelemetry.trace import get_tracer, SpanKind

from .. import postgres_pb2
from .base import Wire


def _wrap_do_execute(fn):
    async def _do_execute(self, query, executor, timeout, retry=True):
        if query.startswith(('SELECT ', 'select ')):
            operation_name = 'SELECT'
        elif query.startswith(('UPDATE ', 'update ')):
            operation_name = 'UPDATE'
        elif query.startswith(('INSERT ', 'insert ')):
            operation_name = 'INSERT'
        else:
            operation_name = 'SQL'
        statement = ' '.join(line.strip() for line in query.splitlines())
        with get_tracer(__name__).start_span(
            operation_name, kind=SpanKind.CLIENT,
        ) as span:
            span.set_attribute("component", "sql")
            span.set_attribute("sql.statement", statement)
            return await fn(self, query, executor, timeout, retry)
    _do_execute.__wrapped__ = True
    return _do_execute


class PoolWire(Wire):
    """

    .. wire:: harness.wires.asyncpg.PoolWire
      :type: input
      :runtime: python
      :config: harness.postgres.Pool
      :requirements: asyncpg

    """
    pool: Pool
    _connect = None
    _connect_params = None

    def configure(self, value: postgres_pb2.Pool):
        assert isinstance(value, postgres_pb2.Pool), type(value)

        self.pool = create_pool(
            host=value.address.host,
            port=value.address.port,
            user=value.username,
            password=value.password,
            database=value.database,
            min_size=value.min_size,
            max_size=value.max_size or 10,
        )

        if not getattr(Connection._do_execute, '__wrapped__', False):
            Connection._do_execute = _wrap_do_execute(Connection._do_execute)

    async def __aenter__(self):
        await self.pool.__aenter__()

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.pool.__aexit__(exc_type, exc_val, exc_tb)
